/***************************************************************************
 *   Copyright (C) 2005 by Sean Harmer                                     *
 *   sh@rama.homelinux.org                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Steet, Fifth Floor, Boston, MA 02110-1301, USA.             *
 ***************************************************************************/
#include "kacllistview.h"

#include <qpainter.h>
#include <qptrlist.h>

#include <klocale.h>
#include <kdebug.h>


#ifdef HAVE_ACL_LIBACL_H
# include <acl/libacl.h>
#endif
#include <pwd.h>
#include <grp.h>

#include "kfileaclpropsplugin.h"
#include "images.h"


KACLListViewItem* findACLEntryByType( KACLListView* pACLListView,
                                      acl_tag_t tagType )
{
    QListViewItemIterator it( pACLListView );
    KACLListViewItem* pItem;
    while ( ( pItem = dynamic_cast<KACLListViewItem*>( it.current() ) ) != 0 )
    {
        ++it;
        if ( pItem->entryType() == tagType )
            return pItem;
    }

    // No entry of the requested type found
    return 0;
}


KACLListViewItem::KACLListViewItem( QListView* parent, KACLEntry* pEntry )
 : KListViewItem( parent ),
   m_pEntry( pEntry )
{
    if ( m_pEntry )
    {
        m_pACLListView = dynamic_cast<KACLListView*>( parent );
        switch ( m_pEntry->m_type )
        {
        case ACL_USER_OBJ:
            setPixmap( 0, m_pACLListView->m_aAvatars[KACLListView::AA_USER_OBJ] );
            setText( 0, "Owner" );
            break;

        case ACL_GROUP_OBJ:
            setPixmap( 0, m_pACLListView->m_aAvatars[KACLListView::AA_GROUP_OBJ] );
            setText( 0, "File Group" );
            break;

        case ACL_OTHER:
            setPixmap( 0, m_pACLListView->m_aAvatars[KACLListView::AA_OTHER] );
            setText( 0, "Other" );
            break;

        case ACL_USER:
            setPixmap( 0, m_pACLListView->m_aAvatars[KACLListView::AA_USER] );
            setText( 0, "User" );
            break;

        case ACL_GROUP:
            setPixmap( 0, m_pACLListView->m_aAvatars[KACLListView::AA_GROUP] );
            setText( 0, "Group" );
            break;

        case ACL_MASK:
            setPixmap( 0, m_pACLListView->m_aAvatars[KACLListView::AA_MASK] );
            setText( 0, "Mask" );
            break;

        default:
            setPixmap( 0, m_pACLListView->m_aAvatars[KACLListView::AA_USER] );
            setText( 0, "Unknown" );
            break;
        }

        // Set the qualifier text
        setText( 1, m_pEntry->m_qualifier );

        // Set the pixmaps for which of the perms are set
        updatePermPixmaps();
    }
}


KACLListViewItem::~ KACLListViewItem()
{

}


acl_tag_t KACLListViewItem::entryType() const
{
    if ( m_pEntry != 0 )
        return m_pEntry->m_type;
    return ACL_UNDEFINED_TAG;
}


void KACLListViewItem::paintCell( QPainter* p, const QColorGroup &cg,
                                  int column, int width, int alignment )
{
    if ( m_pEntry && m_pEntry->isPartialEntry() == true )
    {
        QFont font = p->font();
        font.setItalic( true );
        QColorGroup mycg = cg;
        mycg.setColor( QColorGroup::Text, QColor( 100, 100, 100 ) );

        p->setFont( font );
        KListViewItem::paintCell( p, mycg, column, width, alignment );
    }
    else
    {
        KListViewItem::paintCell( p, cg, column, width, alignment );
    }

    // Draw dotted lines above and below the mask entry if it exists to
    // help distinguish between standard perms and an extended ACL.
    if ( m_pEntry && m_pEntry->m_type == ACL_MASK )
    {
        p->setPen( QPen( Qt::gray, 0, QPen::DotLine ) );
        p->drawLine( 0, 0, width - 1, 0 );
        p->drawLine( 0, height() - 1, width - 1, height() - 1 );
    }
}


void KACLListViewItem::updatePermPixmaps()
{
    if ( m_pEntry )
    {
        acl_perm_t perms = m_pEntry->m_perms;
        acl_perm_t partialPerms = m_pEntry->m_partialPerms;

        if ( perms & ACL_READ )
            setPixmap( 2, m_pACLListView->m_pmTick );
        else if ( partialPerms & ACL_READ )
            setPixmap( 2, m_pACLListView->m_pmPartial );
        else
            setPixmap( 2, QPixmap() );

        if ( perms & ACL_WRITE )
            setPixmap( 3, m_pACLListView->m_pmTick );
        else if ( partialPerms & ACL_WRITE )
            setPixmap( 3, m_pACLListView->m_pmPartial );
        else
            setPixmap( 3, QPixmap() );

        if ( perms & ACL_EXECUTE )
            setPixmap( 4, m_pACLListView->m_pmTick );
        else if ( partialPerms & ACL_EXECUTE )
            setPixmap( 4, m_pACLListView->m_pmPartial );
        else
            setPixmap( 4, QPixmap() );
    }
}


void KACLListViewItem::calcEffectiveRights()
{
    kdDebug() << "KACLListViewItem::calcEffectiveRights()" << endl;
    QString strEffective = QString( "---" );

    // Does a mask entry exist?
    bool bMask = m_pACLListView->hasMaskEntry();
    acl_perm_t perms = m_pEntry->m_perms;
    acl_perm_t partialPerms = m_pEntry->m_partialPerms;

    // Do we need to worry about the mask entry?
    if ( !bMask || entryType() == ACL_USER_OBJ || entryType() == ACL_OTHER )
    {
        // No, the effective perms are just the perms in this entry
        strEffective[0] = ( perms & ACL_READ ) ? 'r' : '-';
        strEffective[1] = ( perms & ACL_WRITE ) ? 'w' : '-';
        strEffective[2] = ( perms & ACL_EXECUTE ) ? 'x' : '-';

        // What about any partial perms?
        if ( partialPerms & ACL_READ )
            strEffective[0] = 'R';
        if ( partialPerms & ACL_WRITE )
            strEffective[1] = 'W';
        if ( partialPerms & ACL_EXECUTE )
            strEffective[2] = 'X';
    }
    else
    {
        acl_perm_t maskPerms = m_pACLListView->maskPermissions();
        acl_perm_t maskPartialPerms = m_pACLListView->maskPartialPermissions();
        //kdDebug() << "mask = " << mask << " perms = " << m_perms << endl;

        strEffective[0] = ( maskPerms & perms & ACL_READ ) ? 'r' : '-';
        strEffective[1] = ( maskPerms & perms & ACL_WRITE ) ? 'w' : '-';
        strEffective[2] = ( maskPerms & perms & ACL_EXECUTE ) ? 'x' : '-';

        // What about any partial perms?
        if ( maskPerms & partialPerms & ACL_READ || // Partial perms on entry
             maskPartialPerms & perms & ACL_READ || // Partial perms on mask
             maskPartialPerms & partialPerms & ACL_READ ) // Partial perms on mask and entry
            strEffective[0] = 'R';
        if ( maskPerms & partialPerms & ACL_WRITE || // Partial perms on entry
             maskPartialPerms & perms & ACL_WRITE || // Partial perms on mask
             maskPartialPerms & partialPerms & ACL_WRITE ) // Partial perms on mask and entry
            strEffective[1] = 'W';
        if ( maskPerms & partialPerms & ACL_EXECUTE || // Partial perms on entry
             maskPartialPerms & perms & ACL_EXECUTE || // Partial perms on mask
             maskPartialPerms & partialPerms & ACL_EXECUTE ) // Partial perms on mask and entry
            strEffective[2] = 'X';
    }

    setText( 5, strEffective );
}


void KACLListViewItem::togglePerm( acl_perm_t perm )
{
    m_pEntry->m_perms ^= perm; // Toggle the perm

    // If the perm is in the partial perms then remove it. i.e. Once
    // a user changes a partial perm it then applies to all selected files.
    if ( m_pEntry->m_partialPerms & perm )
        m_pEntry->m_partialPerms ^= perm;

    // Make sure that all entries have their effective rights calculated if
    // we are changing the ACL_MASK entry.
    if ( entryType() == ACL_MASK )
    {
        m_pACLListView->setMaskPartialPermissions( m_pEntry->m_partialPerms );
        m_pACLListView->setMaskPermissions( m_pEntry->m_perms );
        m_pACLListView->calculateEffectiveRights();
    }

    m_pEntry->setPartialEntry( false );

    calcEffectiveRights();
    updatePermPixmaps();
}


KACLListView::KACLListView( KACL* pACL, QWidget* parent, const char* name )
 : KListView( parent, name ),
   m_pACL( pACL ),
   m_bHasMask( false )
{
    // Add the columns
    addColumn( i18n( "Type" ) );
    addColumn( i18n( "Name" ) );
    addColumn( i18n( "r" ) );
    addColumn( i18n( "w" ) );
    addColumn( i18n( "x" ) );
    addColumn( i18n( "Effective" ) );

    // Load the avatars
    /**  \todo make these static (in KACLListViewItem?) or load from external file? */
    m_aAvatars[AA_USER_OBJ] = QPixmap( qembed_findImage( "user-grey" ) );
    m_aAvatars[AA_GROUP_OBJ] = QPixmap( qembed_findImage( "group-grey" ) );
    m_aAvatars[AA_OTHER] = QPixmap( qembed_findImage( "others-grey" ) );
    m_aAvatars[AA_USER] = QPixmap( qembed_findImage( "user" ) );
    m_aAvatars[AA_GROUP] = QPixmap( qembed_findImage( "group" ) );
    m_aAvatars[AA_MASK] = QPixmap( qembed_findImage( "mask" ) );

    m_pmTick = QPixmap( qembed_findImage( "yes" ) );
    m_pmPartial = QPixmap( qembed_findImage( "yespartial" ) );

    // Turn off sorting
    setSorting( -1 );

    // Make some connections
    connect( this, SIGNAL( clicked( QListViewItem*, const QPoint&, int ) ),
             this, SLOT( entryClicked( QListViewItem*, const QPoint&, int ) ) );

    setACL( pACL );

    setSelectionMode( QListView::Extended );
}


KACLListView::~KACLListView()
{

}


void KACLListView::setACL( KACL* pACL )
{
    //kdDebug() << "KACLListView::setACL()" << endl;
    if ( !pACL )
        return;

    // Remove any entries left over from displaying a previous ACL
    clear();
    m_bHasMask = false;

    // Update which ACL we should use
    m_pACL = pACL;

    // Insert the entries
    QPtrList<KACLEntry>& entries = m_pACL->entries();
    QPtrListIterator<KACLEntry> it( entries );
    KACLEntry* pEntry;
    while ( ( pEntry = it.current() ) != 0 )
    {
        ++it;
        new KACLListViewItem( this, pEntry );
    }

    // Now to sort the list of KACLListViewItems. We show the entries
    // corresponding to the usual POSIX perms first (ACL_USER_OBJ, ACL_GROUP_OBJ,
    // ACL_OTHER), followed by the mask entry (if it exists) and finally the
    // ACL_USER and ACL_GROUP entries.

    // Find the ACL_USER_OBJ, ACL_GROUP_OBJ, and ACL_OTHER entries
    KACLListViewItem* pUserObj = findACLEntryByType( this, ACL_USER_OBJ );
    KACLListViewItem* pGroupObj = findACLEntryByType( this, ACL_GROUP_OBJ );
    KACLListViewItem* pOther = findACLEntryByType( this, ACL_OTHER );
    pGroupObj->moveItem( pUserObj );
    pOther->moveItem( pGroupObj );

    // Do we have any additional ACL_USER or ACL_GROUP entries?
    KACLListViewItem* pMask = findACLEntryByType( this, ACL_MASK );
    if ( pMask != 0 )
    {
        // An ACL_MASK entry exists so there may be some ACL_USER or ACL_GROUP entries
        // Move the mask to after the ACL_OTHER entry
        m_bHasMask = true;
        pMask->moveItem( pOther );

        // Store a pointer to the mask entry so that we can quickly calculate the effective
        // rights for each of the affected ACL entries: ACL_GROUP_OBJ, ACL_USER, and ACL_GROUP.
        m_pMaskEntry = pMask->m_pEntry;
    }

    // Iterate through the entries and move them to after the mask entry if they
    // are of the ACL_USER or ACL_GROUP type.
    QListViewItemIterator it2( this );
    KACLListViewItem* pItem;
    while ( ( pItem = dynamic_cast<KACLListViewItem*>( it2.current() ) ) != 0 )
    {
        ++it2;

        // We may as well calculate the effective perms while we are here
        pItem->calcEffectiveRights();

        if ( pItem->entryType() == ACL_USER || pItem->entryType() == ACL_GROUP )
            pItem->moveItem( pMask );
    }
}


void KACLListView::entryClicked( QListViewItem* pItem, const QPoint& /*pt*/, int col )
{
    if ( pItem != 0 )
    {
        KACLListViewItem* pACLItem = dynamic_cast<KACLListViewItem*>( pItem );
        switch ( col )
        {
            case 2:
                pACLItem->togglePerm( ACL_READ );
                break;
            case 3:
                pACLItem->togglePerm( ACL_WRITE );
                break;
            case 4:
                pACLItem->togglePerm( ACL_EXECUTE );
                break;

            default:
            ; // Do nothing
        }

        // Has the user changed one of the required entries in a default ACL?
        if ( m_pACL->aclType() == ACL_TYPE_DEFAULT &&
             ( col == 2 || col == 3 || col == 4 ) &&
             ( pACLItem->entryType() == ACL_USER_OBJ ||
               pACLItem->entryType() == ACL_GROUP_OBJ ||
               pACLItem->entryType() == ACL_OTHER ) )
        {
            // Mark the required entries as no longer being partial entries.
            // That is, they will get applied to all selected directories.
            KACLListViewItem* pUserObj = findACLEntryByType( this, ACL_USER_OBJ );
            pUserObj->entry()->setPartialEntry( false );

            KACLListViewItem* pGroupObj = findACLEntryByType( this, ACL_GROUP_OBJ );
            pGroupObj->entry()->setPartialEntry( false );

            KACLListViewItem* pOther = findACLEntryByType( this, ACL_OTHER );
            pOther->entry()->setPartialEntry( false );

            update();
        }
    }
}


void KACLListView::calculateEffectiveRights()
{
    QListViewItemIterator it( this );
    KACLListViewItem* pItem;
    while ( ( pItem = dynamic_cast<KACLListViewItem*>( it.current() ) ) != 0 )
    {
        ++it;
        pItem->calcEffectiveRights();
    }
}


acl_perm_t KACLListView::maskPermissions() const
{
    return m_pMaskEntry->m_perms;
}


void KACLListView::setMaskPermissions( acl_perm_t maskPerms )
{
    m_pMaskEntry->m_perms = maskPerms;
}


acl_perm_t KACLListView::maskPartialPermissions() const
{
    return m_pMaskEntry->m_partialPerms;
}


void KACLListView::setMaskPartialPermissions( acl_perm_t maskPartialPerms )
{
    m_pMaskEntry->m_partialPerms = maskPartialPerms;
    calculateEffectiveRights();
}

#include "kacllistview.moc"
