/*
 *   Copyright (C) 2010 Lukas Appelhans <l.appelhans@kde.org>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Library General Public License version 2 as
 *   published by the Free Software Foundation
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */ 
#include "appstats.h"

#include <QDateTime>
#include <nepomuk/resourcemanager.h>
#include <kglobal.h>
#include <kdebug.h>
#include "application.h"
#include "applicationstatistic.h"

AppStats::AppStats()
{
}

AppStats::~AppStats()
{
}

class AppStatsHelper
{
public:
    AppStats global;
};

K_GLOBAL_STATIC(AppStatsHelper, AppStatsPrivate)

AppStats * AppStats::instance()
{
    return &AppStatsPrivate->global;
}

void AppStats::launched(const QString &entry)
{
    if (!Nepomuk::ResourceManager::instance()->init())
        return;
    kDebug() << "Triggered";
    Nepomuk::Application app(entry);
    Nepomuk::ApplicationStatistic statistic;
    foreach(const Nepomuk::ApplicationStatistic &stat, app.applicationStatistics()) {
        if (stat.application() == app)//TODO Pick by activity later on
            statistic = stat;
    }
    if (statistic.lastLaunchDate() == QDateTime()) {
        statistic.setApplication(app);
        app.addApplicationStatistic(statistic);
    }
    statistic.setLastLaunchDate(QDateTime::currentDateTime());
    statistic.setLaunchCount(statistic.launchCount() + 1);
    int score = 0;
    if (statistic.lastLaunchDate().addDays(7) > QDateTime::currentDateTime())
        score = statistic.score();
    else if (statistic.lastLaunchDate().addDays(30) > QDateTime::currentDateTime())
        score = statistic.score() / 1.4;
    else if (statistic.lastLaunchDate().addDays(90) > QDateTime::currentDateTime())
        score = statistic.score() / 2;
    else if (statistic.lastLaunchDate() != QDateTime())
        score = statistic.score() / 4;
    statistic.setScore(qMin(score + 10, 100));
}

int AppStats::score(const QString &entry)
{
    if (!Nepomuk::ResourceManager::instance()->init())
        return 0;
    kDebug() << "Triggered";
    Nepomuk::Application app(entry);
    Nepomuk::ApplicationStatistic statistic;
    foreach(const Nepomuk::ApplicationStatistic &stat, app.applicationStatistics()) {
        if (stat.application() == app)//TODO Pick by activity later on
            statistic = stat;
    } //TODO: Do this better with sparql queries

    int score = 0;
    if (statistic.lastLaunchDate().addDays(7) > QDateTime::currentDateTime())
        score = statistic.score();
    else if (statistic.lastLaunchDate().addDays(30) > QDateTime::currentDateTime())
        score = statistic.score() / 1.4;
    else if (statistic.lastLaunchDate().addDays(90) > QDateTime::currentDateTime())
        score = statistic.score() / 2;
    else if (statistic.lastLaunchDate() != QDateTime())
        score = statistic.score() / 4;
    return score;
}
