/**************************************************************************
*   Copyright (C) 2009 by Ian Reinhart Geiser                             *
*   geiseri@kde.org                                                       *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#ifndef LAUNCHER_H
#define LAUNCHER_H

#include <QFrame>
#include <QCommandLinkButton>

namespace Ui {
    class Launcher;
}
class KDirWatch;

/**
  * Profile Example
  * [Profile]
  * Name=Profile Name
  * Description=Profile description that will appear in the header.
  * Icon=custom.png
  * [KMail]
  * Type=Application
  * Desktop File=kmail
  * [CompanyHomePage]
  * Type=Url
  * Url=http://www.mycompany.com
  * Icon=MyCompanyLogo.png
  * Title=Company Home Page
  * [CustomApplicationWithUrl]
  * Type=Application
  * Desktop File=customapp
  * Icon=CustomIcon.png
  * Title=Tradewinds
  * Description=Data entry application.
  * Url=file://shared/drive/database.db
  */
class ApplicationCommandLinkButton : public QCommandLinkButton
{
    Q_OBJECT
    public:
        ApplicationCommandLinkButton( QWidget *parent = 0 );

        void setDesktopFilePath( const QString &path );
        QString desktopFilePath( ) const;

        void setDesktopStartUrl( const QString &url );
        QString desktopStartUrl( ) const;

    signals:
        void activated( int );

    private slots:
        void executeCommandLink();

    private:
        QString m_desktopFilePath;
        QString m_desktopStartUrl;

};

class Launcher : public QFrame {
    Q_OBJECT
    Q_DISABLE_COPY(Launcher)
public:
    explicit Launcher(QWidget *parent = 0);
    virtual ~Launcher();

signals:
    void popupDone( int code = 1 );

private slots:

    void on_logout_clicked();

    void slotLogout();

    void setupShortcuts();
    void setupTasks();
    void reloadTasks();

protected:
    virtual void changeEvent(QEvent *e);

private:
    void execByServiceType( const QString &app );
    void configureButtonFromDesktopFile( ApplicationCommandLinkButton *button, const QString &desktopFileName );

    KDirWatch *m_watcher;
    Ui::Launcher *m_ui;
};

#endif // LAUNCHER_H
