/*  This file is part of the KDE Libraries
 *  Copyright (C) 2006-2007 by Olivier Goffart (ogoffart at kde.org)
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public License
 *  along with this library; see the file COPYING.LIB.  If not, write to
 *  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 *  Boston, MA 02110-1301, USA.
 */

#include "kcolorpalette.h"
#include <kdebug.h>
#include <kconfig.h>
#include <kglobal.h>
#include <kcomponentdata.h>

class KColorPalette::Private
{ public:
	QHash<QString , QColor> bgcolors;
	QHash<QString , QColor> fgcolors;
	QHash<QString , QString> parents;
	
//	void readKeyColor( const QString &key , const QString &type, KConfig* local_conf , KConfig* global_conf);
};


//-----------------------

KColorPalette::KColorPalette( const QString & appname )
	: d(new Private)
{
    if(appname.isEmpty())
        load( KGlobal::mainComponent().componentName() );
    else
	    load( appname );
}

KColorPalette::KColorPalette( KComponentData* instance )
    : d(new Private)
{
    load( instance->componentName() );
}


KColorPalette::~KColorPalette()
{
	delete d;
}

/*** the global icon loader ***/
Q_GLOBAL_STATIC(KColorPalette, globalColorPalette)

KColorPalette *KColorPalette::global()
{
    return globalColorPalette( );
}


void KColorPalette::load( const QString & appname )
{
	if(appname!="kde")
	{
		d->bgcolors.clear();
		d->fgcolors.clear();
		load("kde");
	}
	KConfig global_conf("data" , appname + '/' + appname + ".kcolorsrc");
	KConfig local_conf( "kcolorsrc"  );
	local_conf.setGroup( appname );
	foreach(QString key , global_conf.groupList() )
	{
		if(key.startsWith('!'))
			continue;
		
		global_conf.setGroup(key);
		QColor c=local_conf.readEntry( key + "-BackgroundColor" , QColor()  );
		if( !c.isValid() )
			c=global_conf.readEntry( "BackgroundColor", QColor()  );
		if( c.isValid() )
			d->bgcolors.insert(key, c);
		
		c=local_conf.readEntry( key + "-ForegroundColor" , QColor()  );
		if( !c.isValid() )
			c=global_conf.readEntry( "ForegroundColor" , QColor() );
		if( c.isValid() )
			d->fgcolors.insert(key, c);
		
		QString parent=global_conf.readEntry( "Parent" , QString() );
		if(!parent.isEmpty())
			d->parents.insert(key, parent );
	}
}

QColor KColorPalette::backgroundColor( const QString & name ) const
{
	QString n2=name;
	forever
	{
		if(d->bgcolors.contains(n2))
			return d->bgcolors.value(n2);
		
		if(d->parents.contains(n2))
			n2=d->parents.value(n2);
		else
		{
			kError() << "There is no background color for name "<< name << endl;
			return QColor();
		}
	}
}

QColor KColorPalette::foregroundColor( const QString & name ) const
{
	QString n2=name;
	forever
	{
		if(d->fgcolors.contains(n2))
			return d->fgcolors.value(n2);
		
		if(d->parents.contains(n2))
			n2=d->parents.value(n2);
		else
		{
			kError() << "There is no background color for name "<< name << endl;
			return QColor();
		}
	}
}



