/*  This file is part of the KDE Libraries
 *  Copyright (C) 2006-2007 by Olivier Goffart (ogoffart at kde.org)
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public License
 *  along with this library; see the file COPYING.LIB.  If not, write to
 *  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 *  Boston, MA 02110-1301, USA.
 */

#ifndef KCOLORPALETTE_H
#define KCOLORPALETTE_H

#include <kdeui_export.h>
#include <QtGui/QColor>

class KComponentData;

/**
 * This class manage the palette of color you can use in your application
 * @author Olivier Goffart
 */
class KDEUI_EXPORT KColorPalette
{
public:
	explicit KColorPalette(const QString &appname = QString());
    explicit KColorPalette(KComponentData* instance);
	~KColorPalette();
    
    
    /**
     * Returns the global color palette initialized with the global KComponentData.
     * Therefore you must have a KComponentData instantiated before calling this.
     * @return global color palette
     */
    static KColorPalette* global();


	/**
	 * @return the color that you can use in the background with the name @p name
	 */
	QColor backgroundColor(const QString& name) const;
	/**
	 * @return the color that you can use in the foreground with the name @p name
	 */
	QColor foregroundColor(const QString& name) const;
	
	/**
	 * reload the color palette
	 * @param appname is the application name, if empty, kapp->instanceName is used
	 */
	void load(const QString &appname = QString());
	
private:
	class Private;
	Private * const d;
	Q_DISABLE_COPY(KColorPalette);
};

#endif
