/**
 *  Copyright (C) 2004 Benjamin C Meyer (ben+thistest at meyerhome dot net)
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public License
 *  along with this library; see the file COPYING.LIB.  If not, write to
 *  the Free Software Foundation, Inc., 51 Franklin Steet, Fifth Floor,
 *  Boston, MA 02110-1301, USA.
 */

/**
 * TEST: Unecessary usage of this->
 * 
 * When compiled with optimizations the two times should be the same
 * When compiled without optimizations this-> will be about 1/2 the speed 
 */

#include <sys/time.h>
#include <qstringlist.h>

#define LOOP 100000

/**
 * Read in paramaters and output a maze line by line.
 */
int main( int argc, char *argv[] ) {
	struct timeval tv1;
	struct timeval tv2;
	QStringList list;
	int a;
	for(uint i=0; i < LOOP; i++){
		list.append(QString("%1").arg(i));
	}
	
	a=0;
	gettimeofday(&tv1, NULL);
	for ( QStringList::Iterator it = list.begin(); it != list.end(); it++ ) {
		a++;
	}
  gettimeofday(&tv2, NULL);
	qDebug("post:  %d, %d", (tv2.tv_usec-tv1.tv_usec), a);

	gettimeofday(&tv1, NULL);
	a=0;
	for ( QStringList::Iterator it = list.begin(); it != list.end(); ++it ) {
		a++;
	}
	gettimeofday(&tv2, NULL);
	qDebug("pre:   %d, %d", (tv2.tv_usec-tv1.tv_usec), a);

	a=0;
	gettimeofday(&tv1, NULL);
	QStringList::Iterator end = list.end();
	for ( QStringList::Iterator it = list.begin(); it != end; ++it ) {
	  a++;
	}
	gettimeofday(&tv2, NULL);
	qDebug("endout:%d, %d", (tv2.tv_usec-tv1.tv_usec), a);
	
	a=0;
	gettimeofday(&tv1, NULL);
	const QStringList::const_iterator cend = list.constEnd();
	for ( QStringList::const_iterator it = list.begin(); it != cend; ++it ) {
	  a++;
	}
	gettimeofday(&tv2, NULL);
	qDebug("const: %d, %d", (tv2.tv_usec-tv1.tv_usec), a);
	
	a=0;
	gettimeofday(&tv1, NULL);
	const QStringList::const_iterator start = list.constBegin();
	for ( QStringList::const_iterator it = list.end(); it != start; --it ) {
	  a++;
	}
	gettimeofday(&tv2, NULL);
	qDebug("const--: %d, %d", (tv2.tv_usec-tv1.tv_usec), a);

	a=0;
	gettimeofday(&tv1, NULL);
	for(int i=LOOP;i--;){
		a++;
	}
	gettimeofday(&tv2, NULL);
	qDebug("i-- as comparison: %d, %d", (tv2.tv_usec-tv1.tv_usec), a);
	
	a=0;
	gettimeofday(&tv1, NULL);
	for(int i=LOOP; i>0; --i){
		a++;
	}
	gettimeofday(&tv2, NULL);
	qDebug("i>0 as comparison: %d, %d", (tv2.tv_usec-tv1.tv_usec), a);
	
	a=0;
	gettimeofday(&tv1, NULL);
	for(int i=0; i<LOOP; ++i){
		a++;
	}
	gettimeofday(&tv2, NULL);
	qDebug("i<LOOP as comparison: %d, %d", (tv2.tv_usec-tv1.tv_usec), a);
	
	return 0;
}

