/*
 *  Copyright (C) 2008 by Tom Albers <tomalbers@kde.nl>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public License
 *  along with this library; see the file COPYING.LIB.  If not, write to
 *  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 *  Boston, MA 02110-1301, USA.
 **/

#ifndef plugin_bookmarksync_h
#define plugin_bookmarksync_h

#include <kparts/plugin.h>
#include <kurl.h>

class KDirWatch;
class KActionMenu;
class KJob;
class QTimer;

class SettingsDialog;

class PluginBookmarkSync : public KParts::Plugin
{
    Q_OBJECT

public:
    PluginBookmarkSync( QObject* parent, const QVariantList & );
    virtual ~PluginBookmarkSync();
    enum Type { Save = 0, Fetch };

public slots:
    void slotConfigure();
    void slotSave();
    void slotFetch();

private slots:
    void slotDone( KJob* );
    void slotGetLocation();
    void slotSaveMaybe();
    void slotSaveDoIt();
    void slotAutoFetch();

private:
    void copy( PluginBookmarkSync::Type );
    int getLocalSize();

    KActionMenu*  m_menu;
    KAction* m_fetchAction;
    KAction* m_storeAction;

    QWidget* m_parent;
    SettingsDialog* m_settings;
    KDirWatch* m_watch;
    KConfig* m_config;
    QTimer *m_saveTimer;

    KUrl m_location;
    bool m_autoSave;
    bool m_autoFetch;
    int m_localSize;

    bool m_autoFetchInProgress;
};

#endif
