/*
 *  Copyright (C) 2009 by Tom Albers <tomalbers@kde.nl>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public License
 *  along with this library; see the file COPYING.LIB.  If not, write to
 *  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 *  Boston, MA 02110-1301, USA.
 **/

// local
#include "plugin_clearhttpauth.h"

// KDE
#include <KActionCollection>
#include <KGenericFactory>
#include <kwallet.h>

// Qt
#include <QDBusInterface>

typedef KGenericFactory<PluginClearHttpAuth> PluginClearHttpAuthFactory;
K_EXPORT_PLUGIN( PluginClearHttpAuthFactory( "clearhttpauth" ) )

PluginClearHttpAuth::PluginClearHttpAuth( QObject* parent, const QStringList & )
        : Plugin( parent )
{
    // parent
    if ( !parent || !parent->inherits( "KHTMLPart" ) )
        return;
    m_part = qobject_cast<KHTMLPart *>( parent );
    //m_parent = part->widget();

    m_clear = new KAction( i18n( "Clear HTTP Authentication" ), actionCollection() );
    actionCollection()->addAction( "action clearhttpauth", m_clear );
    connect( m_clear, SIGNAL( triggered() ), this, SLOT( slotClearHttpAuth() ) );
}

PluginClearHttpAuth::~PluginClearHttpAuth()
{
}

void PluginClearHttpAuth::slotClearHttpAuth()
{
    kDebug() << "Clear it!" << m_part->url();
    QDBusInterface dbus( "org.kde.kded", "/modules/kpasswdserver","org.kde.KPasswdServer" );
    dbus.call( "removeAuthInfo", m_part->url().host(), m_part->url().protocol(), QString() );

    // remove it from the wallet too
    KWallet::Wallet* wallet = KWallet::Wallet::openWallet(
                                  KWallet::Wallet::NetworkWallet(), m_part->widget()->winId() );

    // no wallet?
    if ( !wallet )
        return;

    // set folder or if no password folder exit.
    if ( !wallet->hasFolder( KWallet::Wallet::PasswordFolder() ) ) {
        delete wallet;
        return;
    }
    wallet->setFolder( KWallet::Wallet::PasswordFolder() );

    // get all entries and find the one that applies...
    const QStringList list = wallet->entryList();
    QString key;
    foreach( const QString& item, list ) {
        kDebug() << "Considering: " << item;
        if ( item.startsWith( m_part->url().protocol()+'-'+m_part->url().host() ) ) {
            key = item;
            break;
        }
    }

    // did we find the correct entry?
    if ( key.isEmpty() ) {
        kDebug() << "No entry found";
        delete wallet;
        return;
    }

    kDebug() << "Deleting:" << key;
    wallet->removeEntry( key );
    delete wallet;

    // Reload the page.
    KParts::OpenUrlArguments args;
    args.setReload( true );
    m_part->setArguments( args );
    m_part->openUrl( m_part->url() );
}

#include "plugin_clearhttpauth.moc"
