#include "imagebrowser.h"

#include <KLocale>
#include <KDirModel>
#include <KDirLister>
#include <KFileItemDelegate>
#include <KIconLoader>
#include <KLineEdit>

#include <QListView>
#include <QVBoxLayout>

#include <Nepomuk/Query/FileQuery>
#include <Nepomuk/Query/QueryParser>
#include <Nepomuk/Query/AndTerm>
#include <Nepomuk/Query/ResourceTypeTerm>

#include "nfo.h"


ImageBrowser::ImageBrowser()
    : KMainWindow()
{
    QWidget* w = new QWidget(this);
    m_queryEdit = new KLineEdit(w);
    connect(m_queryEdit, SIGNAL(returnPressed()), this, SLOT(slotQueryChanged()) );
    QListView* view = new QListView( w );
    QVBoxLayout* layout = new QVBoxLayout(w);
    layout->addWidget(m_queryEdit);
    layout->addWidget(view);
    setCentralWidget(w);

    view->setIconSize( QSize(KIconLoader::SizeMedium, KIconLoader::SizeMedium) );

    KFileItemDelegate* delegate = new KFileItemDelegate(view);
    delegate->setShowInformation(KFileItemDelegate::InformationList() << KFileItemDelegate::FriendlyMimeType );
    view->setItemDelegate(delegate);

    m_model = new KDirModel( this );
    view->setModel(m_model);

    Nepomuk::Query::FileQuery query;
    query.setTerm( Nepomuk::Query::ResourceTypeTerm( Nepomuk::Vocabulary::NFO::RasterImage() ) );
//    query.setLimit(-1);
    m_model->dirLister()->openUrl(query.toSearchUrl());
}

ImageBrowser::~ImageBrowser()
{
}


void ImageBrowser::slotQueryChanged()
{
    Nepomuk::Query::FileQuery query = Nepomuk::Query::QueryParser::parseQuery( m_queryEdit->text() );
    query.setTerm( Nepomuk::Query::AndTerm( query.term(), Nepomuk::Query::ResourceTypeTerm( Nepomuk::Vocabulary::NFO::RasterImage() ) ) );
    m_model->dirLister()->openUrl(query.toSearchUrl());
}

#include "imagebrowser.moc"
