#include "backupthread.h"
#include <QDateTime>
#include <QFile>
#include <QTextStream>

#include <KDebug>

#include <Soprano/Model>
#include <Soprano/Serializer>
#include <Soprano/QueryResultIterator>
#include <Soprano/StatementIterator>
#include <Soprano/Statement>
#include <Soprano/PluginManager>

#include <Nepomuk/ResourceManager>

BackupThread::BackupThread(QObject *parent)
    : QThread(parent)
{
    // connect the configuration
    readConfig();
}

BackupThread::~BackupThread()
{
}

void BackupThread::run()
{
    QFile file(m_path);
    if (!file.open(QFile::WriteOnly | QFile::Text))
    {
        kDebug() << "Cannot write to file!";
        return;
    }
    QTextStream stream(&file);

    QString query = "select distinct ?s ?p ?o ?g where { graph ?g { ?s ?p ?o . } . OPTIONAL { { ?g <http://www.semanticdesktop.org/ontologies/2007/01/19/nie#dataGraphFor> ?x . } UNION { ?g <http://www.semanticdesktop.org/ontologies/2007/08/15/nrl#coreGraphMetadataFor> ?g2 . ?g2 <http://www.semanticdesktop.org/ontologies/2007/01/19/nie#dataGraphFor> ?x . } UNION { ?g <http://www.strigi.org/fields#indexGraphFor> ?x . } UNION { ?g <http://www.semanticdesktop.org/ontologies/2007/08/15/nrl#coreGraphMetadataFor> ?g2 . ?g2 <http://www.strigi.org/fields#indexGraphFor> ?x . } } . FILTER(!BOUND(?x)) . FILTER(?g != <urn:crappyinference:inferredtriples>) . }";

    const Soprano::Serializer *serializer = Soprano::PluginManager::instance()->discoverSerializerForSerialization(Soprano::SerializationNQuads);

    Soprano::QueryResultIterator resultIterator = Nepomuk::ResourceManager::instance()->mainModel()->executeQuery(query, Soprano::Query::QueryLanguageSparql);
    if (resultIterator.isGraph())
    {
        Soprano::StatementIterator iterator = resultIterator.iterateStatements();
        serializer->serialize(iterator, stream, Soprano::SerializationNQuads);
    }
    else
    {
        Soprano::StatementIterator iterator = resultIterator.iterateStatementsFromBindings( "s", "p", "o", "c" );
        serializer->serialize(iterator, stream, Soprano::SerializationNQuads);
    }

    // FIXME - remove the statements loaded from ontology files!
}

void BackupThread::readConfig()
{
    // get the file to which to save the backup
    // get whether or not to save strigi output - in case the filesystem does not change, but the repository is emptied
    m_path = QString("/mnt/rest/nepomuk_%1.n4").arg(QDateTime::currentDateTime().toString("ddMMyyyy_hhmm"));
    m_strigi = false;
}

void BackupThread::configChanged()
{
    readConfig();
}
