/*
    This file is part of the Nepomuk KDE project.
    Copyright (C) 2010  Vishesh Handa <handa.vish@gmail.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "identifier.h"
#include "syncfile.h"
#include "mergerequest.h"
#include "changelog.h"
#include "identificationthread.h"
#include "identifieradaptor.h"

#include <QtDBus/QDBusConnection>

#include <Soprano/Serializer>
#include <Soprano/PluginManager>
#include <Soprano/Util/SimpleStatementIterator>

#include <KDebug>

Nepomuk::Identifier::Identifier( QObject* parent )
    : QObject(parent)
{
    m_thread = new IdentificationThread( this );

    connect( m_thread, SIGNAL( identified( int, QUrl, QUrl ) ),
             this, SLOT( identified( int, QUrl, QUrl ) ) );
    connect( m_thread, SIGNAL( notIdentified( int, QList<Soprano::Statement> ) ),
             this, SLOT( notIdentified( int, QList<Soprano::Statement> ) ) );
    connect( m_thread, SIGNAL( processed( Nepomuk::Sync::ChangeLog ) ),
             this, SIGNAL( processed( Nepomuk::Sync::ChangeLog ) ) );

    //Register DBus interface
    new IdentifierAdaptor( this );
    QDBusConnection dbus = QDBusConnection::sessionBus();
    dbus.registerObject( QLatin1String("/identifier"), this );
}


Nepomuk::Identifier::~Identifier()
{
    m_thread->stop();
    m_thread->wait();
}


bool Nepomuk::Identifier::identify(int id, const QString& oldUri, const QString& newUri)
{
    kDebug() << newUri;
    return m_thread->identify( id, QUrl(oldUri), QUrl(newUri) );
}

int Nepomuk::Identifier::process(const Nepomuk::Sync::SyncFile& sf)
{
    return m_thread->process( sf );
}

bool Nepomuk::Identifier::ignore(int id, const QString& url, bool ignoreSubDirectories)
{
    return m_thread->ignore( id, QUrl( url ), ignoreSubDirectories );
}

void Nepomuk::Identifier::test()
{
    emit notIdentified( 0, QString() );
    emit identified(0, QString(), QString() );
}

void Nepomuk::Identifier::notIdentified(int id, const QList< Soprano::Statement >& st)
{
    const Soprano::Serializer* serializer = Soprano::PluginManager::instance()->discoverSerializerForSerialization( Soprano::SerializationNQuads );

    Soprano::Util::SimpleStatementIterator it( st );
    QString ser;
    QTextStream stream( &ser );
    serializer->serialize( it, stream, Soprano::SerializationNQuads );

    emit notIdentified( id, ser );
}


void Nepomuk::Identifier::identified(int id, const QUrl& oldUri, const QUrl& newUri)
{
    emit identified( id, oldUri.toString(), newUri.toString() );
}
