/*
    <one line to give the program's name and a brief idea of what it does.>
    Copyright (C) 2010  Vishesh Handa <handa.vish@gmail.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "backupsyncservice.h"
#include "diffgenerator.h"
#include "identifier.h"
#include "merger.h"
#include "syncmanager.h"
#include "backupmanager.h"
#include "dbusoperators.h"

#include "syncfile.h"
#include "changelog.h"

#include <KDebug>

#include <kpluginfactory.h>
#include <kpluginloader.h>

NEPOMUK_EXPORT_SERVICE( Nepomuk::BackupSyncService, "nepomukbackupsync" )

Nepomuk::BackupSyncService::BackupSyncService( QObject* parent, const QList< QVariant >& )
	: Service(parent)
{
    kDebug();

    m_diffGenerator = new DiffGenerator( this );
    m_identifier = new Identifier( this );
    m_merger = new Merger( this );

    m_syncManager = new SyncManager( m_identifier, m_merger, this );
    m_backupManager = new BackupManager( m_identifier, m_merger, this );

    // IMPORTANT : We've used "Nepomuk::ChangeLog" in the string cause in the slots, signals, and
    // connect statement we're using Nepomuk::ChangeLog, NOT ChangeLog
    qRegisterMetaType<Nepomuk::ChangeLog>("Nepomuk::ChangeLog");

    registerMetaTypes();
    connect( m_identifier, SIGNAL( processed( Nepomuk::ChangeLog ) ),
             m_merger, SLOT( process( Nepomuk::ChangeLog ) ) );
}

Nepomuk::BackupSyncService::~BackupSyncService()
{
}

void Nepomuk::BackupSyncService::test()
{
    //QUrl url("/home/vishesh/syncnew");

    m_identifier->test();
    /*
    LogFile lf;
    lf.load(QUrl("file:///home/vishesh/ident.trig"));
    SyncFile::create( lf, url );
    */
    /*

    SyncFile sf( url );

    LogFile lf = sf.logFile();
    foreach( const Record & r, lf.allRecords() ) {
        kDebug() << r.st;
    }

    kDebug();
    kDebug();
    kDebug() << "Identification File";
    foreach( const Soprano::Statement & st, sf.identificationFile().allStatements() ) {
        kDebug() << st;
    }
    */

    //Sync::SyncFile sf( url );
    //m_identifier->process( sf );
//    m_identifier->process( sf );
//     m_identifier->process( sf );
//     m_identifier->process( sf );
//     m_identifier->process( sf );
//     m_identifier->process( sf );

}

Nepomuk::Identifier* Nepomuk::BackupSyncService::identifier() const
{
    return m_identifier;
}

Nepomuk::Merger* Nepomuk::BackupSyncService::merger() const
{
    return m_merger;
}

Nepomuk::DiffGenerator* Nepomuk::BackupSyncService::diffGenerator() const
{
    return m_diffGenerator;
}

Nepomuk::SyncManager* Nepomuk::BackupSyncService::syncManager() const
{
    return m_syncManager;
}

Nepomuk::BackupManager* Nepomuk::BackupSyncService::backupManager() const
{
    return m_backupManager;
}




#include "backupsyncservice.moc"
