/*
 *
 * $Id: sourceheader 511311 2006-02-19 14:51:05Z trueg $
 *
 * This file is part of the Nepomuk KDE project.
 * Copyright (C) 2006 Sebastian Trueg <trueg@kde.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * See the file "COPYING" for the exact licensing terms.
 */

#include "statementlistmodel.h"
#include "file.h"

#include <nepomuk/class.h>

#include <KLocale>
#include <KIconLoader>

#include <QtCore/QVector>
#include <QtGui/QPixmap>

class Nepomuk::StatementListModel::Private
{
public:
    QVector<Soprano::Statement> statements;
};


Nepomuk::StatementListModel::StatementListModel( QObject* parent )
  : QAbstractItemModel( parent ),
    d( new Private() )
{
}


Nepomuk::StatementListModel::~StatementListModel()
{
    delete d;
}


QList<Soprano::Statement> Nepomuk::StatementListModel::statements() const
{
    return d->statements.toList();
}


void Nepomuk::StatementListModel::clear()
{
    d->statements.clear();
    reset();
}


void Nepomuk::StatementListModel::setStatements( const QList<Soprano::Statement>& statements )
{
    d->statements = statements.toVector();
    reset();
}


void Nepomuk::StatementListModel::addStatement( const Soprano::Statement& statement )
{
    beginInsertRows( QModelIndex(), d->statements.count(), d->statements.count() + 1 );
    d->statements.append( statement );
    endInsertRows();
}


void Nepomuk::StatementListModel::addStatements( const QList<Soprano::Statement>& statements )
{
    beginInsertRows( QModelIndex(), d->statements.count(), d->statements.count() + statements.count() );
    d->statements += statements.toVector();
    endInsertRows();
}


int Nepomuk::StatementListModel::columnCount( const QModelIndex& parent ) const
{
    Q_UNUSED( parent );
    return 5;
}


QVariant Nepomuk::StatementListModel::headerData( int section, Qt::Orientation orientation, int role ) const
{
    Q_UNUSED( orientation );

    if( role == Qt::DisplayRole ) {
        switch( section ) {
        case 0:
            return i18n( "Subject" );
        case 1:
            return i18n( "Predicate" );
        case 2:
            return i18n( "Object" );
        case 3:
            return i18n( "Language" );
        case 4:
            return i18n( "Datatype" );
        }
    }

    return QVariant();
}


QVariant Nepomuk::StatementListModel::data( const QModelIndex& index, int role ) const
{
    if( index.isValid() ) {
        switch( role ) {
        case Qt::DisplayRole:
            switch( index.column() ) {
            case 0:
                return d->statements[index.row()].subject().toString();
            case 1:
                return d->statements[index.row()].predicate().toString();
            case 2:
                return d->statements[index.row()].object().toString();
            case 3:
                return d->statements[index.row()].object().language();
            case 4:
                return d->statements[index.row()].object().dataType().toString();
            }

        case Qt::ToolTipRole:
            if ( const Nepomuk::Class* c = Nepomuk::Class::load( d->statements[index.row()].object().uri() ) ) {
                return c->label();
            }
        }
    }

    return QVariant();
}


QModelIndex Nepomuk::StatementListModel::index( int row, int column, const QModelIndex& parent ) const
{
    Q_UNUSED( parent );

    if( row < d->statements.count() )
        return createIndex( row, column );
    else
        return QModelIndex();
}


QModelIndex Nepomuk::StatementListModel::parent( const QModelIndex& index ) const
{
    Q_UNUSED( index );
    return QModelIndex();
}


int Nepomuk::StatementListModel::rowCount( const QModelIndex& index ) const
{
    // no child elements yet
    if( index.isValid() )
        return 0;
    else
        return d->statements.count();
}

#include "statementlistmodel.moc"
