/*
Copyright (c) 2010 Oszkar Ambrus <aoszkar@gmail.com>
Copyright (C) 2010 Sebastian Trueg <trueg@kde.org>

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) version 3, or any
later version accepted by the membership of KDE e.V. (or its
successor approved by the membership of KDE e.V.), which shall
act as a proxy defined in Section 6 of version 3 of the license.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "resourcelistmodel.h"
#include "resourceitemdelegate.h"

#include <KDebug>
#include <KIcon>

#include <Nepomuk/Query/Result>
#include <Nepomuk/Resource>
#include <KExtendableItemDelegate>

Q_DECLARE_METATYPE(Nepomuk::Resource)

class Nepomuk::ResourceListModel::Private
{
public:
    QList<Nepomuk::Query::Result> resources;//should we hold pointers to resources?

};

Nepomuk::ResourceListModel::ResourceListModel(QObject * parent) :
        ResourceModel(parent),
        d(new Private())
{
}

Nepomuk::ResourceListModel::~ResourceListModel()
{
    delete d;
}

QModelIndex Nepomuk::ResourceListModel::indexForResource( const Resource& res ) const
{
    Q_ASSERT( res.isValid() );
    // FIXME: performance
    int i = 0;
    QList<Nepomuk::Query::Result>::const_iterator end = d->resources.constEnd();
    for ( QList<Nepomuk::Query::Result>::const_iterator it = d->resources.constBegin(); it != end; ++it ) {
        if ( it->resource() == res ) {
            return index( i, 0 );
        }
        ++i;
    }

    return QModelIndex();
}

Nepomuk::Resource Nepomuk::ResourceListModel::resourceForIndex( const QModelIndex& index ) const
{
    if ( index.isValid() && index.row() < d->resources.count() ) {
        return d->resources[index.row()].resource();
    }
    else {
        return Resource();
    }
}

void Nepomuk::ResourceListModel::setResources(const QList<Nepomuk::Query::Result>& resources)
{
    d->resources = resources;
    reset();
}

QVariant Nepomuk::ResourceListModel::data(const QModelIndex& index, int role) const
{
    if (index.row() < 0 || index.row() >= d->resources.count()) {
        kDebug() << "Invalid index:" << index;
        return QVariant();
    }

    if ( role == Qt::EditRole ) { //pass Resource to editor
        return qVariantFromValue(d->resources[index.row()].resource());
    }
    else if ( role == KExtendableItemDelegate::ShowExtensionIndicatorRole  ) {
        return false;//true;
    }
    else if ( role == ResourceItemDelegate::ResourceUriRole ) {
        return d->resources[index.row()].resource().uri();
    }
    else if ( role == ResourceItemDelegate::ResourceTypeRole ) {
        return Nepomuk::Types::Class( d->resources[index.row()].resource().type() ).label();
    }
    else if ( role == ResourceItemDelegate::ResourceDescriptionRole ) {
        return d->resources[index.row()].resource().description();
    }
    else {
        return ResourceModel::data( index, role );
    }
}

int Nepomuk::ResourceListModel::rowCount(const QModelIndex& parent) const
{
    if (parent.isValid()) {
        return 0;
    } else {
        return d->resources.count();
    }
}

QModelIndex Nepomuk::ResourceListModel::index(int row, int column, const QModelIndex& parent) const
{
    if (row >= 0 && row < rowCount(parent) && column == 0) {
        return createIndex(row, column, 0);
    } else {
        return QModelIndex();
    }
}

bool Nepomuk::ResourceListModel::removeRows(int row, int count, const QModelIndex &parent)
{
    if (count <= 0 || row < 0 || (row + count) > rowCount(parent))
        return false;

    beginRemoveRows(QModelIndex(), row, row + count - 1);

    for (int r = 0; r < count; ++r)
        d->resources.removeAt(row);

    endRemoveRows();

    return true;
}
