/* 
 *
 * $Id: sourceheader 511311 2006-02-19 14:51:05Z trueg $
 *
 * This file is part of the Nepomuk KDE project.
 * Copyright (C) 2006 Sebastian Trueg <trueg@kde.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * See the file "COPYING" for the exact licensing terms.
 */

#include "tripleservicetest.h"

#include <QtGlobal>
#include <QDateTime>

#include <knepomuk/registry.h>
#include <knepomuk/service.h>
#include <knepomuk/services/tripleservice.h>
#include <knepomuk/services/statementlistiterator.h>

using namespace Nepomuk::Backbone;
using namespace Nepomuk::Backbone::Services;
using namespace Nepomuk::Backbone::Services::RDF;

static Statement s_theStatement = Statement( Node( "http://purl.org/net/dagnele/", NodeResource ),
					     Node( "http://purl.org/dc/elements/1.1/creator", NodeResource ),
					     Node( "Daniele Galdi", NodeLiteral ) );

TripleServiceTest::TripleServiceTest()
  : QObject()
{
  m_registry = new Registry( this );
}


void TripleServiceTest::initTestCase()
{
  m_service = m_registry->discoverServiceByType( "http://nepomuk.semanticdesktop.org/services/storage/rdf/Triple" );
  Q_ASSERT( m_service != 0 );
}

void TripleServiceTest::testAddGraph()
{
  TripleService tsw ( m_service );
  
  if( tsw.addGraph( "http://nepomuk.org/kde#uri:nepomuk:69" ) ) {
    QFAIL( "addGraph failed." );
  }
}

void TripleServiceTest::testListGraphs()
{
  TripleService tsw ( m_service );
  QStringList graphs = tsw.listGraphs();

  QStringListIterator iter( graphs );
  while ( iter.hasNext() )
  {
    QString graph = iter.next();
    QWARN( (const char *) graph.toLatin1().data() );
  }
  //QCOMPARE( iter.next(), QString("http://nepomuk.org/kde#uri:nepomuk:1") );
}

void TripleServiceTest::testAddStatement()
{
  TripleService tsw( m_service );
  if( tsw.addStatement( "http://nepomuk.org/kde#uri:nepomuk:69", s_theStatement ) ) {
    QFAIL( "addStatement failed." );
  }
}

void TripleServiceTest::testListStatements()
{
  TripleService tsw( m_service );
  StatementList list = tsw.listStatements( "http://nepomuk.org/kde#uri:nepomuk:69", Statement( s_theStatement.subject,
								 s_theStatement.predicate,
								 Node() ), 10 );
  QCOMPARE( 1, list.statements.size() );
  QCOMPARE( s_theStatement, list.statements.first() );

  qDebug() << "list.count = " << list.count;
  qDebug() << "list.listId = " << list.listId;
  qDebug() << "list.statements.size() = " << list.statements.size();
  
  StatementListIterator it( list, &tsw );
  //QCOMPARE( list.statements.count(), it.count() );
  int cnt = 0;
  while( it.hasNext() ) {
    qDebug() << "hasNext()";
    ++cnt;
    RDF::Statement st = it.next();
    qDebug() << "Subject: " << st.subject.value;
    //QCOMPARE( s_theStatement, st );
  }
  //QCOMPARE( cnt, it.count() );
}

void TripleServiceTest::testRemoveStatement()
{
  TripleService tsw( m_service );
  if( tsw.removeStatement( "http://nepomuk.org/kde#uri:nepomuk:69", s_theStatement ) ) {
    QFAIL( "removeStatement failed." );
  }
}

QTEST_MAIN(TripleServiceTest)

#include "tripleservicetest.moc"
