/***************************************************************************
 *   Copyright (C) 2005,2006,2007 by Siraj Razick                          *
 *   siraj@kdemail.net                                                     *
 *                                                                         *
 *   Copyright (C) 2008 by Shawn Starr <shawn.starr@rogers.com>            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#include "ciavc.h"

#include <QPainter>
#include <QGraphicsScene>
#include <QGraphicsLinearLayout>
#include <QGraphicsWidget>
#include <QGraphicsProxyWidget>
#include <QtWebKit/QWebView>
#include <KConfigDialog>
#include <KLineEdit>

#include <plasma/theme.h>

CiaVc::CiaVc(QObject *parent, const QVariantList &args)
    : Plasma::Applet(parent, args)
{
    setHasConfigurationInterface(true);
    setAspectRatioMode(Plasma::IgnoreAspectRatio);
    m_layout = 0;
    m_form = 0;
    resize(600,600);

}

void CiaVc::init() 
{
    KConfigGroup cg = config();
    m_currentProject = cg.readEntry("project", "KDE");
    m_ciavcEngine = dataEngine("ciavc");

    m_browser = new QWebView();
    m_proxy = new QGraphicsProxyWidget(this);
    m_proxy->setWidget(m_browser);
    m_browser->setHtml(i18n("Please wait, getting CIA commit information..."));
    m_proxy->show();
    m_layout = new QGraphicsLinearLayout(Qt::Vertical);
    m_layout->addItem(m_proxy);
    setLayout(m_layout);
    connect(m_ciavcEngine, SIGNAL(sourceAdded(QString)), this, SLOT(sourceAdded(QString)));
    sourceAdded(m_currentProject);
}

void CiaVc::sourceAdded(const QString& source)
{
    m_browser->setHtml(i18n("Please wait, getting CIA commit information..."));
    m_ciavcEngine->connectSource(source, this);
}

bool CiaVc::revisionLessThan(const QString &s1, const QString &s2)
{
    static QRegExp re("::(?:.*)r<b>([0-9]+)");
    int r1 = 0;
    int r2 = 0;

    if (re.indexIn(s1) != -1) {
        r1 = re.cap(1).toInt(); 
    }
    if (re.indexIn(s2) != -1) {
        r2 = re.cap(1).toInt();
    }
    return r1 > r2;
}

void CiaVc::dataUpdated(const QString &source, const Plasma::DataEngine::Data &data)
{ 
    Q_UNUSED(source)
    QStringList htmlCommitList;
    QString htmlContent;
     
    // Set body colour
    htmlCommitList.append("<body bgcolor=\"darkgrey\">");
    foreach (QVariant item, data) {
       htmlCommitList += item.toString();
    }
    qSort(htmlCommitList.begin(), htmlCommitList.end(), revisionLessThan);

    htmlContent = htmlCommitList.join("");
    // Set some nicer colours.
    htmlContent.replace("#DDD", "#ffffff");

    // Save space by killing double newlines
    htmlContent.replace( QRegExp("(<br>\\n*)+"), "<br>" );
    htmlContent.append("</body>");
    // Set some default background colour
    m_browser->setHtml(htmlContent);
    m_browser->update();
}

void CiaVc::createConfigurationInterface(KConfigDialog *parent)
{
    QWidget *widget = new QWidget();
    ui.setupUi(widget);
    parent->setButtons(KDialog::Ok | KDialog::Cancel | KDialog::Apply);
    parent->addPage(widget, parent->windowTitle(), icon());
    ui.rssCIAFeed->setTrapReturnKey(true);
    connect(parent, SIGNAL(okClicked()), this, SLOT(configAccepted()));
    connect(parent, SIGNAL(applyClicked()), this, SLOT(configAccepted()));
    connect(ui.rssCIAFeed, SIGNAL(returnPressed()), this, SLOT(configAccepted()));
    ui.rssCIAFeed->setText(m_currentProject);
}

void CiaVc::configAccepted()
{
    KConfigGroup cg = config();
    m_currentProject = ui.rssCIAFeed->text();
    if  (!ui.rssCIAFeed->text().isEmpty()) {
         cg.writeEntry("project", m_currentProject);
    }
    // Clear out all other sources
    foreach (const QString source, m_ciavcEngine->sources()) {
         m_ciavcEngine->disconnectSource(source, this);
    }

    sourceAdded(ui.rssCIAFeed->text());
    update();
}

void CiaVc::paintInterface(QPainter *p, const QStyleOptionGraphicsItem *option, const QRect &contentsRect)
{
    Q_UNUSED(p);
    Q_UNUSED(option);
    Q_UNUSED(contentsRect);
}

#include "ciavc.moc"
