/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   Copyright (C) 2008 by Dennis Nienhüser <earthwings@gentoo.org>        *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#include "cellfactory.h"

#include "abstractcell.h"
#include "eventcell.h"
#include "iconcell.h"
#include "labelcell.h"
#include "lineeditcell.h"
#include "picturecell.h"
#include "statuscell.h"

#include "contactactions.h"

AbstractCell *CellFactory::construct(const QString &id,
		const QVariantMap &properties, QGraphicsWidget *parent)
{
	if (id == "photo")
	{
		return new IconCell(id, "system-users", properties, parent);
	}
	else if (id == "name")
	{
		return new LabelCell(id, properties, parent);
	}
	else if (id == "email")
	{
		return new LabelCell(id, properties, parent);
	}
	else if (id == "instant_messaging_id")
	{
		return new LabelCell(id, properties, parent);
	}
	else if (id == "instant_messaging_status")
	{
		LabelCell *cell = new LabelCell(id, properties, parent);
		cell->setAnimationsEnabled(true);
		return cell;
	}
	else if (id == "instant_messaging_quickchat")
	{
		// TODO: This is too Kopete specific, as well as the name of the class confusing
		LineEditCell *cell = new LineEditCell("name", "instant_messaging_can_send_message", properties, parent);
		connect(cell, SIGNAL(inputFinished(QString,QString)), ContactActions::self(), SLOT(sendMessageKopete(QString,QString)));
		return cell;
	}
	else if ( id == "events" )
	{
	    EventCell *cell = new EventCell(properties, parent);
	    return cell;
	}
    else if ( id == "status" )
    {
        StatusCell *cell = new StatusCell(properties, parent);
        return cell;
    }

	kDebug() << "Don't know how to create a cell for " << id;
	return new LabelCell(id, properties, parent);
}

QStringList CellFactory::defaultCells()
{
	return QStringList() << "photo" << "name" << "status";
}

QStringList CellFactory::knownCells()
{
	return QStringList() << "photo" << "name" << "email" << "instant_messaging_id"
			<< "instant_messaging_status" << "instant_messaging_quickchat" << "events" << "status";
}
