/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   Copyright (C) 2008 by Dennis Nienhüser <earthwings@gentoo.org>        *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#include "contactwidget.h"

#include <QtGui/QPainter>
#include <QtGui/QGraphicsLinearLayout>
#include <QtCore/QRectF>
#include <QtCore/QList>

#include <kicon.h>
#include <klocale.h>
#include <kdebug.h>

#include "abstractcell.h"
#include "cellfactory.h"

class ContactWidgetPrivate
{
public:

	ContactWidgetPrivate(QGraphicsLayoutItem * parent = 0)
	{
		m_layout = new QGraphicsLinearLayout(Qt::Horizontal, parent);
	}

	QGraphicsLinearLayout* layout()
	{
		Q_ASSERT(m_layout);
		return m_layout;
	}

	QList<AbstractCell*>& cells()
	{
		return m_cells;
	}

private:
	QGraphicsLinearLayout *m_layout;
	QList<AbstractCell*> m_cells;
};

ContactWidget::ContactWidget(const QStringList &cells, const QVariantMap &properties,
		QGraphicsItem *parent) :
	QGraphicsWidget(parent), d(new ContactWidgetPrivate())
{
	setLayout(d->layout());

	foreach( const QString &cell, cells)
	{
		AbstractCell *cellWidget = CellFactory::construct(cell, properties, 0);
		Q_ASSERT(cellWidget);
		d->layout()->addItem(cellWidget);
		d->cells().append(cellWidget);
	}
}

ContactWidget::~ContactWidget()
{
	for (int i = d->layout()->count() - 1; i >= 0; --i)
	{
		d->layout()->removeAt(i);
	}

	foreach(AbstractCell* cell, d->cells())
	{
		cell->deleteLater();
	}
        delete d;
}

void ContactWidget::propertiesChanged(const QVariantMap &properties)
{
	// Just forward to all cells
	foreach(AbstractCell* cell, d->cells())
	{
		cell->propertiesChanged(properties);
	}
}
