/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   Copyright (C) 2008 by Dennis Nienhüser <earthwings@gentoo.org>        *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#ifndef LABEL_CELL_H_
#define LABEL_CELL_H_

#include "abstractcell.h"

#include <Plasma/Label>

/**
 * Uses a Plasma::Label to display strings
 */
class LabelCell: public AbstractCell
{
Q_OBJECT

public:
	/**
	 * Constructor
	 * @param id Property key identifying the value that should be displayed
	 * @param properties Contacts properties
	 * @param parent Parent widget
	 */
	LabelCell(const QString id, const QVariantMap &properties, QGraphicsWidget *parent)			;

	/** Destructor */
	virtual~ LabelCell();

	/**
	 * Reimplemented from AbstractCell
	 */
	virtual void propertiesChanged(const QVariantMap &properties);

	/**
	 * Toggle animation setting. If true, changes to the displayed string will be animated by a "typing" animation.
	 * @param enabled True if animations should be enabled
	 */
	void setAnimationsEnabled(bool enabled);

private Q_SLOTS:
	/**
	 * Typing animation
	 */
	void animatedTextChange();

private:
	/**
	 * Change the text displayed. Will trigger animation if enabled.
	 */
	void setText(const QString &text);

	/** Property key identifying the value that should be displayed */
	const QString m_id;

	/** Label used for displaying */
	Plasma::Label* m_label;

	/** Current text, used for animation purposes */
	QString m_text;

	/** Animation progress, used for animation purposes */
	QString m_subText;

	/** Stores animation preference */
	bool m_animate;
};

#endif // LABEL_CELL_H_
