/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   Copyright (C) 2008 by Dennis Nienhüser <earthwings@gentoo.org>        *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#include "marblecontactswidget.h"

#include <QtGui/QPainter>
#include <QtGui/QStandardItem>
#include <QtGui/QGraphicsGridLayout>
#include <QtGui/QGridLayout>

#include <kdebug.h>

#include <marble/AbstractFloatItem.h>
#include <marble/RenderPlugin.h>
#include <marble/MarbleWidgetInputHandler.h>


MarbleContactsWidget::MarbleContactsWidget(const QString &themeID, QGraphicsItem *parent) :
	GlobalWidget(parent)
{
	QGraphicsGridLayout *layout = new QGraphicsGridLayout();

	m_marble = new AnnotatedMarbleWidget();
	m_marble->setRadius(6000);
	m_marble->setMapThemeId(themeID);
	m_marble->setShowCompass(false);
	m_marble->setShowScaleBar(false);
	m_marble->setRecenterOnLocationChange(true);
	m_marble->inputHandler()->setMouseButtonPopupEnabled(Qt::LeftButton, false);

	// Hide the overview map
	QList<Marble::AbstractFloatItem *> floatItemList = m_marble->floatItems();
	QList<Marble::AbstractFloatItem*>::const_iterator itr;
	for (itr = floatItemList.constBegin(); itr != floatItemList.constEnd(); ++itr)
	{
		if ( (*itr)->nameId() == "overviewmap" )
		{
			(*itr)->setVisible(false);
		}

		if ( (*itr)->nameId() == "navigation")
		{
			(*itr)->setEnabled(true);
		}
	}

	QGraphicsProxyWidget *proxy = new QGraphicsProxyWidget(this);
	proxy->setWidget(m_marble);

	layout->addItem(proxy,0,0);
	setLayout(layout);
}

MarbleContactsWidget::~MarbleContactsWidget()
{
	/** @todo: cleanup marble in some way? */
}

void MarbleContactsWidget::propertiesChanged(const QString &id,
		const QVariantMap &properties)
{
	m_marble->propertiesChanged(id, properties);
}

void MarbleContactsWidget::setRecenterOnLocationChange(bool enabled)
{
	m_marble->setRecenterOnLocationChange(enabled);
}
