/***************************************************************************
 *   Copyright (C) 2009 by Matthieu Gallien <matthieu_gallien@yahoo.fr>    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#include "status_notifier_item_model.h"

#include <KIcon>

#include <QtGlobal>

StatusNotifierItemModel::StatusNotifierItemModel(Plasma::DataEngine *engine, QObject *parent) :
    QAbstractListModel(parent),
    m_engine(engine),
    m_itemsData(),
    m_indexFromItemName(),
    m_itemNameFromIndex()
{
    connect(m_engine, SIGNAL(sourceAdded(const QString&)),
            this, SLOT(itemAdded(const QString&)));
    connect(m_engine, SIGNAL(sourceRemoved(const QString&)),
            this, SLOT(itemRemoved(const QString&)));
    m_engine->connectAllSources(this);
}

StatusNotifierItemModel::~StatusNotifierItemModel()
{
    const QStringList &sources = m_engine->sources();
    for (QStringList::const_iterator itSource = sources.begin(); itSource != sources.end(); ++itSource) {
        m_engine->disconnectSource(*itSource, this);
    }
}

QVariant StatusNotifierItemModel::data(const QModelIndex &index, int role) const
{
    const QString &itemName = m_itemNameFromIndex[index.row()];

    switch (role)
    {
        case Qt::DisplayRole:
            return m_itemsData[itemName]["Title"];
        case Qt::DecorationRole:
            return m_itemsData[itemName]["Icon"];
    }

    return QVariant();
}

int StatusNotifierItemModel::rowCount(const QModelIndex &index) const
{
    Q_UNUSED(index);
    if (index.isValid()) {
        return 0;
    } else {
        return m_itemNameFromIndex.size();
    }
}

void StatusNotifierItemModel::itemAdded(const QString &taskName)
{
    m_engine->connectSource(taskName, this);
}

void StatusNotifierItemModel::itemRemoved(const QString &taskName)
{
    int oldIndex = m_indexFromItemName[taskName];
    beginRemoveRows(QModelIndex(), oldIndex, oldIndex);
    m_engine->disconnectSource(taskName, this);
    m_itemsData.remove(taskName);
    m_indexFromItemName.remove(taskName);
    removeIndex(oldIndex);
    m_itemNameFromIndex.resize(m_itemNameFromIndex.size() - 1);
    endRemoveRows();
}

void StatusNotifierItemModel::dataUpdated(const QString &itemName, const Plasma::DataEngine::Data &itemData)
{
    if (m_itemsData.contains(itemName)) {
        m_itemsData[itemName] = itemData;
        emit dataChanged(index(m_indexFromItemName[itemName], 0), index(m_indexFromItemName[itemName], 0));
    } else {
        addItem(itemName, itemData);
    }
}

void StatusNotifierItemModel::addItem(const QString &itemName, const Plasma::DataEngine::Data &itemData)
{
    int newIndex = m_itemNameFromIndex.size();
    beginInsertRows(QModelIndex(), newIndex, newIndex);
    m_indexFromItemName[itemName] = newIndex;
    m_itemNameFromIndex.resize(newIndex + 1);
    m_itemNameFromIndex[newIndex] = itemName;
    m_itemsData[itemName] = itemData;
    endInsertRows();
}

void StatusNotifierItemModel::removeIndex(int removedIndex)
{
    for (int index = removedIndex; index < m_itemNameFromIndex.size() - 1; ++index) {
        m_indexFromItemName[m_itemNameFromIndex[index + 1]] = index;
        m_itemNameFromIndex[index] = m_itemNameFromIndex[index + 1];
    }
}
