/*
 *   Copyright (C) 2007 Ivan Cukic <ivan.cukic+kde@gmail.com>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser/Library General Public License version 2,
 *   or (at your option) any later version, as published by the Free
 *   Software Foundation
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU Lesser/Library General Public License for more details
 *
 *   You should have received a copy of the GNU Lesser/Library General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef EMBEDDED_STATES_ICON_H_
#define EMBEDDED_STATES_ICON_H_

#include <QGraphicsWidget>

#include <KIcon>
#include <Plasma/PanelSvg>
#include <QMap>

class StatesIcon : public QGraphicsWidget {
    Q_OBJECT

public:
    enum State {
        Offline,
        Starting,
        Running,
        Active
    };

    StatesIcon(QGraphicsItem * parent = 0, Qt::WindowFlags wFlags = 0);
    ~StatesIcon();

    void registerState(State state, KIcon icon);

    void setState(State state);
    State state() const;

    void setBackgroundSvg(const QString & svg);

    void setIconSize(const QSize & size);
    QSize iconSize() const;

    // override
    virtual void paint(QPainter * painter, const QStyleOptionGraphicsItem * option, QWidget * widget = 0);

    virtual void hoverEnterEvent(QGraphicsSceneHoverEvent * event);
    virtual void hoverLeaveEvent(QGraphicsSceneHoverEvent * event);

    virtual void mousePressEvent(QGraphicsSceneMouseEvent * event);
    virtual void mouseReleaseEvent(QGraphicsSceneMouseEvent * event);

Q_SIGNALS:
    void clicked();

private:
    enum ButtonState {
        Normal,
        Disabled,
        Pressed,
        Hovered
    };

    void setSvgPrefix();

    State m_state;
    ButtonState m_buttonState;
    QMap < State, KIcon > m_states;
    QSize m_iconSize;
    Plasma::PanelSvg m_svg;
};

#endif // EMBEDDED_STATES_ICON_H_

