/***************************************************************************
 *   Copyright © 2009 Reza Fatahilah Shah <rshah0385@kireihana.com>        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/
#include "infowidget.h"
#include <KDebug>
#include <QPainter>
#include <QTimer>
#include <QStyleOptionGraphicsItem>
#include <Plasma/Svg>
#include <Plasma/FrameSvg>
#include <Plasma/Theme>
#include <Plasma/Animator>

InfoWidget::InfoWidget(QGraphicsWidget *parent,Plasma::Svg *svg)
        : QGraphicsWidget(parent),mCountDownRunning(false)
{
    setZValue(1000);
    mFrameSvg = new Plasma::FrameSvg(this);
    mFrameSvg->setImagePath("widgets/background");
    mFrameSvg->setCacheAllRenderedFrames(true);
    mFrameSvg->setEnabledBorders(Plasma::FrameSvg::AllBorders);
    
    mSvg = svg;
    mMaxCountDownSecond = 5;
    mOpacity=0.0;
    mNumberFont=Plasma::Theme::defaultTheme()->font(Plasma::Theme::DefaultFont);
    mNumberFont.setPointSize(24);
    
    mCountDownTimer = new QTimer(this);
    connect(mCountDownTimer, SIGNAL(timeout()), this, SLOT(slotCountDownTimer()));
}

InfoWidget::~InfoWidget()
{
}

void InfoWidget::setSize(QSizeF size)
{
    mSize = size;
    mWidgetRect = QRectF(QPointF(0,0),this->mSize);
    mCircleSize= mSize.height()-2;
    mCircleRect = QRectF((mSize.width()-mCircleSize)/2,(mSize.height()-mCircleSize)/2,mCircleSize,mCircleSize);
    QFontMetrics m(mNumberFont);
    mNumberPosition = QPointF(mCircleRect.x() + (mCircleRect.size().width() / 2) - m.width("0") / 2,
                        (mCircleRect.size().height() / 2) + m.ascent() / 2);
    mFrameSvg->resizeFrame(mSize);
}

void InfoWidget::startCountDown(int counter)
{
    /*Steps:
        1. Fade in
        2. start counter
        3. Fade out
    */
    //Plasma::Animator::self()->moveItem(this,Plasma::Animator::FastSlideInMovement,QPoint(50,50));
    Plasma::Animator::self()->customAnimation(20, 500,Plasma::Animator::EaseInCurve,
                                              this, "appearAnimation");
    mMaxCountDownSecond = counter;
    mCountDownSecond = mMaxCountDownSecond;
    mCountDownRunning = true;
    mCountDownTimer->start(1000);
}

void InfoWidget::appearAnimation(qreal animationvalue)
{
    //kDebug() << "animationvalue= " << animationvalue ;
    mOpacity = animationvalue;
}

void InfoWidget::disappearAnimation(qreal animationvalue)
{
    //kDebug() << "animationvalue= " << animationvalue ;
    mOpacity = 1 - animationvalue;
    if (mOpacity < 0.0) {
        mOpacity = 0.0;
    }
}

void InfoWidget::slotCountDownTimer()
{
    kDebug() << "Count Down " << mCountDownSecond;
    if (mCountDownSecond == 0) {
        mCountDownTimer->stop();
        mCountDownRunning = false;
        emit countDownFinished();
        Plasma::Animator::self()->customAnimation(20, 500,
                                                  Plasma::Animator::EaseInCurve,
                                                  this, "disappearAnimation");
    } else {
        update();
        mCountDownSecond--;
    }
}

void InfoWidget::paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget)
{
    Q_UNUSED(option)
    Q_UNUSED(widget)

    painter->setOpacity(mOpacity);
    mFrameSvg->paintFrame(painter);
    
    if (mCountDownRunning) {
        painter->setOpacity(1.0);
        painter->setFont(mNumberFont);
        painter->drawText(mNumberPosition,QString::number(mCountDownSecond));
    }
}

#include "infowidget.moc"