/***************************************************************************
 *   Copyright © 2009 Reza Fatahilah Shah <rshah0385@kireihana.com>        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/
#ifndef KEREN_H
#define KEREN_H

#include <QImage>
#include <Plasma/PopupApplet>

class QTimer;
class KerenConfigDialog;
class PreviewWidget;
class VideoGst;
class SvgButton;

namespace Plasma
{
    class PushButton;
    class Frame;
    class IconWidget;
    class Svg;
    class Label;
	class SvgWidget;
}

class KerenAppletPrivate;
class KerenApplet : public Plasma::PopupApplet
{
    Q_OBJECT
public:
    friend class KerenAppletPrivate;
    KerenApplet(QObject *parent, const QVariantList &args);
    ~KerenApplet();

    virtual QGraphicsWidget *graphicsWidget();
    void init();

protected:
    virtual void popupEvent (bool show);
    virtual void constraintsEvent(Plasma::Constraints constraints);

public slots:
    void createConfigurationInterface(KConfigDialog *parent);

private Q_SLOTS:
    void slotTakePicture();
	void slotTakeVideo();
    void slotCountDownFinished();
    void slotScreenshotTaken(QImage *image);
    
protected Q_SLOTS:
    void configAccepted();
    void configCancelled();

private:
    enum CaptureType { Picture, Video };
    void loadConfig();
    void setupPipeline();
    void createLayout();
    void checkHomeDirectoryStucture();
    void suspendCapture();
    void resumeCapture();
    QString createNewCaptureFilename();

private:
    KerenConfigDialog *mConfigDialog;
    QGraphicsWidget *mWidget;
    PreviewWidget *mPreviewWidget;
    VideoGst *mVideoGst;
    CaptureType mCaptureType;
    SvgButton * mPictureButton;
    SvgButton * mVideoButton;
    Plasma::Frame *mControlFrame;
    Plasma::Svg *mSvg;
    QImage mImage;
    QString mVideoDevicePath;
    QSize mVideoFrameSize;
    QSize mVideoFrameRate;
    
    QString mCapturePath;
    QString mAlbumName;
    bool mIsRecordingVideo;
};

K_EXPORT_PLASMA_APPLET(keren, KerenApplet)
#endif
