/***************************************************************************
 *   Copyright © 2009 Reza Fatahilah Shah <rshah0385@kireihana.com>        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/
#ifndef CONFIG_DIALOG_H
#define CONFIG_DIALOG_H
#include <QSharedPointer>

#include "ui_videoSettings.h"
#include "ui_audioSettings.h"
#include "libqtgstreamer/qgstdeclarations.h"

class QLabel;
class QTimer;
class VideoWidget;
class KPageWidgetItem;

typedef struct _GstElement GstElement;

class AudioInputDevice;
typedef QSharedPointer<AudioInputDevice> AudioInputDevicePtr;

class AudioInputDevice;
typedef QSharedPointer<AudioInputDevice> AudioInputDevicePtr;

class VideoCaptureBin;
typedef QSharedPointer<VideoCaptureBin> VideoCaptureBinPtr;

namespace KGstDevices
{
    class DeviceManager;
    class DevicesModel;
}

class KerenConfigDialog : public KDialog
{
    Q_OBJECT
public:
    KerenConfigDialog(QWidget *parent = 0);
    ~KerenConfigDialog();

    void stop();
    void setVideoFrameSize(const QSize &videoFrameSize);
    void setVideoFrameRate(const QSize &videoFrameRate);
    QString videoDevicePath() const;
    QSize videoFrameSize() const;
    QSize videoFrameRate() const;
    Ui::videoSettings videoSettingsUi;
    Ui::audioSettings audioSettingsUi;
    QWidget *videoSettings;
    QWidget *audioSettings;

private Q_SLOTS:
    void slotPageChanged(KPageWidgetItem* currentPage,KPageWidgetItem* previousPage);
    void slotAudioInputDeviceActivated(int index);
    void slotAudioTestCapture();
    void slotVideoDeviceActivated(int index);
    void slotVideoResolutionActivated(int index);
    void slotVideoFrameRateActivated(int index);
    void slotBrightnessSliderChanged(int value);
    void slotContrastSliderChanged(int value);
    void slotSaturationSliderChanged(int value);
    void slotHueSliderChanged(int value);

private:
    void initAudioDevice();
    void initVideoDevice();
    void restartPreview();
    void updateResolutionList();
    void updateFrameRateList();

private:
    KGstDevices::DeviceManager *mDeviceManager;
    KGstDevices::DevicesModel *mAudioInputModel;
    KGstDevices::DevicesModel *mVideoInputModel;
    QtGstreamer::QGstPipelinePtr mVideoPipeline;
    QtGstreamer::QGstPipelinePtr mAudioPipeline;
    VideoCaptureBinPtr mVideoCaptureBin;
    VideoWidget *mVideoWidget;
    
    //audio related
    AudioInputDevicePtr mAudioInputDevice;
    QString mAudioInputDriver;
    QString mAudioSinkDriver;
    QString mAudioInputPath;
    
    //video related
    QSize mVideoFrameRate;
    QSize mVideoFrameSize;
    QString mVideoInputPath;
    QString mVideoInputDriver;
};

#endif