/*
 * This file is part of the KDE project
 * Copyright (C) 2009 Eva Brucherseifer <eva.brucherseifer@basyskom.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef KILLSWITCHDEVICE_H
#define KILLSWITCHDEVICE_H

#include <QtCore/QObject>
#include <QtDBus/QDBusMessage>
#include <QtDBus/QDBusError>


class KillSwitchManager;
class KillSwitchDevicePrivate;



struct ChangeDescription
{
    QString key;
    bool added;
    bool removed;
};


class KillSwitchDevice : public QObject
{
    Q_OBJECT

public:

    //     RFKILL_STATE_SOFT_BLOCKED = 0,   /* Radio output blocked */
    //     RFKILL_STATE_UNBLOCKED    = 1,   /* Radio output allowed */
    //     RFKILL_STATE_HARD_BLOCKED = 2,   /* Output blocked, non-overrideable */
    enum State { softblocked = 0, unblocked = 1, hardblocked=2 };


    KillSwitchDevice(const QString &udi);
    virtual ~KillSwitchDevice();

    virtual QString udi() const;
    virtual QString parentUdi() const;

    virtual QString vendor() const;
    virtual QString product() const;
//    virtual QString icon() const;

    virtual QString type() const;
    virtual int state() const;
    virtual QString name() const;
    virtual QString access_method() const;

    //virtual State power();
    virtual bool setPower(bool poweron);

    virtual QVariant property(const QString &key) const;
    virtual bool queryDeviceInterface(const QStringList &cap_list) const;

   
Q_SIGNALS:
    void propertyChanged(const QMap<QString,int> &changes);
    void conditionRaised(const QString &condition, const QString &reason);

Q_SIGNALS:
    void switchPressed(const QString &udi);
    void switchDone(); //Solid::ErrorType error, QVariant errorData, const QString &udi);

private Q_SLOTS:
    void slotPropertyModified(int count, const QList<ChangeDescription> &changes);
    void slotCondition(const QString &condition, const QString &reason);
    void slotDBusReply(const QDBusMessage &reply);
    void slotDBusError(const QDBusError &error);

private:
    KillSwitchDevicePrivate *d;
};


#endif // KILLSWITCHDEVICE_H

