// Qt
#include <QPainter>
#include <QSizeF>
#include <QGraphicsProxyWidget>
#include <QGraphicsLinearLayout>
// KDE
#include <KLocale>
#include <KMimeType>
#include <KConfigDialog>
#include <KConfigGroup>
#include <KServiceTypeTrader>
#include <Plasma/Svg>
#include <Plasma/Theme>
#include <plasma/widgets/iconwidget.h>
#include <Plasma/ScrollWidget>
#include <KActionCollection>
#include <KDebug>
//Project
#include "kpartapplet.h"

#define LOAD_PART_WIDGET if ( part ) \
{ \
  part->openUrl( m_url ); \
  partWidget = part->widget(); \
  m_part = part; \
}

//BEGIN INIT

KPartApplet::KPartApplet(QObject *parent, const QVariantList &args)
    : Plasma::Applet(parent, args)
{
  setBackgroundHints(DefaultBackground);
  setHasConfigurationInterface(true);
  resize(400, 400);
  m_nopart = true;
}

KPartApplet::~KPartApplet()
{
  if (hasFailedToLaunch()) 
  {
      // Do some cleanup here
  }
  else
  {
      // Save settings
  }
}

void KPartApplet::reloadConfiguration()
{
  m_url = KUrl(m_cg.readEntry("URL", "file:///home/"));
  m_partName = m_cg.readEntry("Part", "");
}

void KPartApplet::init()
{
  m_cg = config();
  reloadConfiguration();
  loadKPartsList();

  m_layout = new QGraphicsLinearLayout(Qt::Vertical, this);
  if(m_partName == "")
  {
    setConfigurationRequired(true);
    return;
  }
  else
  {
    setConfigurationRequired(false);
  }
  
  m_url_edit = new Plasma::LineEdit();
  m_url_edit->setText(m_url.path());
  connect(m_url_edit, SIGNAL(returnPressed()), this, SLOT(urlChanged()));
  
  m_part_widget = loadKPart();
  
  m_toolbox = new Plasma::ScrollWidget();
  loadToolbox();
  
  m_layout->insertItem(0, m_toolbox);
  m_layout->setStretchFactor(m_toolbox, 2);
  m_layout->insertItem(3, m_url_edit);
  m_layout->setStretchFactor(m_url_edit, 0);
  m_layout->insertItem(1, m_part_widget);
  m_layout->setStretchFactor(m_part_widget, 50);
  setLayout(m_layout);
  m_nopart = false;
}
//END INIT

//BEGIN COMMAND-METHODS
void KPartApplet::updateUI()
{
  reloadConfiguration();
  
  m_url_edit->setText(m_url.path());
  m_part->openUrl(m_url);
  /*part_widget = loadKPart();
  m_layout->removeAt(0);
  m_layout->insertItem(0, part_widget);*/
}
//END COMMAND-METHODS

//BEGIN LOAD-METHODS

QGraphicsProxyWidget* KPartApplet::loadKPart()
{  
  const QList<KService::Ptr> services = m_parts.keys();
  KService::Ptr partService;
  foreach(const KService::Ptr service, services)
  {
    if (service.data()->library() == m_partName)
    {
      partService = service;
      break;
    }
  }

  QWidget* partWidget;
  PartType type = m_parts.value(partService);
  QString libName = partService.data()->library();
  KPluginLoader loader(libName.toLocal8Bit());
  KPluginFactory* fac = loader.factory();
  if (!fac)
  {
    QLabel* label = new QLabel(i18n("The library '%1' could not be loaded.", partService.data()->name()));
    label->setAlignment(Qt::AlignCenter);
    partWidget = label;
    setConfigurationRequired(true);
  }
  else 
  {
    if (type == ReadWrite)
    {
      KParts::ReadWritePart *part = fac->create<KParts::ReadWritePart>();
      LOAD_PART_WIDGET
    } else
    {
      KParts::ReadOnlyPart *part = fac->create<KParts::ReadOnlyPart>();
      LOAD_PART_WIDGET
    }
  }
  QGraphicsProxyWidget* wid = new QGraphicsProxyWidget();
  wid->setWidget(partWidget);
  return wid;
}

void KPartApplet::loadToolbox()
{
  QGraphicsLinearLayout *button_layout = new QGraphicsLinearLayout(Qt::Horizontal);
  const QList<QAction*> al = m_part->actionCollection()->actions();
  for(int i = 0; i<al.length(); i++)
  {
    Plasma::IconWidget* icon = new Plasma::IconWidget();
    icon->setAction(al[i]);
    icon->setMaximumSize(48, 48);
    m_toolbox->setMaximumHeight(icon->maximumHeight() * 1.5);
    button_layout->addItem(icon);
  }
  QGraphicsWidget *wid = new QGraphicsWidget();
  wid->setLayout(button_layout);
  m_toolbox->setWidget(wid);
}

void KPartApplet::loadKPartsList()
{
  KServiceTypeTrader *trader = KServiceTypeTrader::self();
  KService::List readOnlyPartsList = trader->query("KParts/ReadOnlyPart");
  KService::List readWritePartsList = trader->query("KParts/ReadWritePart");
  foreach (KService::Ptr service, readOnlyPartsList)
  {
    if (readWritePartsList.contains(service))
    {
      readWritePartsList.removeOne(service);
    }
    else
    {
      if (!partIsAlreadyListed(service))
      {
        m_parts.insert(service, ReadOnly);
      }
    }
  }
  foreach (KService::Ptr service, readWritePartsList)
  {
    if (!partIsAlreadyListed(service))
    {
      m_parts.insert(service, ReadWrite);
    }
  }
}

void KPartApplet::createConfigurationInterface(KConfigDialog* parent)
{
  QWidget *widget = new QWidget(parent);
  m_ui.setupUi(widget);
  parent->addPage(widget, i18n("General"));
  connect(parent, SIGNAL(applyClicked()), this, SLOT(configAccepted()));
  connect(parent, SIGNAL(okClicked()), this, SLOT(configAccepted()));
  /* Load part list */
  const QList<KService::Ptr> keys = m_parts.keys();
  foreach (const KService::Ptr service, keys)
  {
    m_ui.combo_parts->addItem(service.data()->name(), service.data()->library());
  }
  m_ui.combo_parts->setCurrentIndex(m_ui.combo_parts->findData(m_cg.readEntry("Part")));
}
//END LOAD-METHODS

//BEGIN GETTERS(intern)

bool KPartApplet::partIsAlreadyListed(KService::Ptr service)
{
  const QList<KService::Ptr> services = m_parts.keys();
  foreach (KService::Ptr key, services)
  {
    if (key.data()->library() == service.data()->library())
    {
      return true;
    }
  }
  return false;
}

//END GETTERS(intern)

//BEGIN SLOTS

void KPartApplet::urlChanged()
{
  m_cg.writeEntry("URL", m_url_edit->text());
  updateUI();
}

void KPartApplet::configAccepted()
{ 
  //bool changed = false; TODO unused
  m_cg.writeEntry("Part", m_ui.combo_parts->itemData(m_ui.combo_parts->currentIndex()));
  m_cg.sync();
  reloadConfiguration();
  
  kDebug() << "nopart:" << m_nopart;
  if (!m_nopart)
  {
    //updateUI();
    kDebug() << "let's reload the part" << m_partName;
    m_part_widget = loadKPart();
    m_layout->removeAt(1);
    m_layout->insertItem(1, m_part_widget);
  }
  else
    init();
}
//END SLOTS

#include "kpartapplet.moc"