/***********************************************************************************
* Copyright (C) 2009 by Marco Martin <notmart@gmail.com>
* Copyright (C) 2009 by Michal Dutkiewicz <emdeck@gmail.com>
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*
***********************************************************************************/

#include "TrackListDBusHandler.h"
#include "TrackListAdaptor.h"
#include "MiniPlayerApplet.h"

#include <QList>

#include <KUrl>

TrackListDBusHandler::TrackListDBusHandler(MiniPlayerApplet *parent) : QObject(parent)
{
    m_player = parent;

    setObjectName("TrackListDBusHandler");

    new TrackListAdaptor(this);

    QDBusConnection::sessionBus().registerObject("/TrackList", this);
}

int TrackListDBusHandler::AddTrack(const QString& url, bool playImmediately)
{
    Phonon::MediaSource source(url);

    if (source.type() != Phonon::MediaSource::Invalid)
    {
        m_player->addToPlayList(KUrl::List(KUrl(url)), false);

        if (playImmediately)
        {
            m_player->play(KUrl(url));
        }

        emit TrackListChange(m_player->currentPlayList().count());

        return 0;
    }
    else
    {
        return -1;
    }
}

void TrackListDBusHandler::DelTrack(int index)
{
    if (index < m_player->currentPlayList().count())
    {
        m_player->removeFromPlayList(KUrl::List(m_player->currentPlayList().at(index)));

        emit TrackListChange(m_player->currentPlayList().count());
    }
}

int TrackListDBusHandler::GetCurrentTrack()
{
    return m_player->currentPlayList().indexOf(KUrl(m_player->mediaObject()->currentSource().url()));
}

int TrackListDBusHandler::GetLength()
{
    return m_player->currentPlayList().count();
}

QVariantMap TrackListDBusHandler::GetMetadata(int position)
{
    QVariantMap ret;

    if (position < 0 || position > (m_player->currentPlayList().size() - 1))
    {
        return ret;
    }

    Phonon::MediaObject *mediaObject = new Phonon::MediaObject(this);
    mediaObject->setCurrentSource(Phonon::MediaSource(m_player->currentPlayList().at(position)));

    QVariantMap metaData = m_player->metaData(mediaObject);

    delete mediaObject;

    return metaData;
}

void TrackListDBusHandler::SetLoop(bool enable)
{
    m_player->setRepeatMode(enable?RepeatPlayList:NoRepeat);
}

void TrackListDBusHandler::SetRandom(bool enable)
{
    m_player->setRandomTrack(enable);
}

#include "TrackListDBusHandler.moc"
