/***************************************************************************
 *   Copyright (C) 2008 by Matej Svejda <mata@aw-modell.at>                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#include "peachy_applet.h"

#include "peachy_icon.h"
#include "peachy_layout.h"
#include "peachy_animator.h"

#include <QPainter>
#include <QFontMetrics>
#include <QSizeF>
#include <QGraphicsGridLayout>

#include <KDebug>

K_EXPORT_PLASMA_APPLET(peachy_applet, PeachyApplet)

PeachyApplet::PeachyApplet(QObject *parent, const QVariantList &args)
	: Plasma::Applet(parent, args)
{
    setBackgroundHints(TranslucentBackground);
}


PeachyApplet::~PeachyApplet()
{
	if (hasFailedToLaunch()) {
		// Do some cleanup here
	} else {
		// Save settings
	}
}

void PeachyApplet::init()
{
	m_animator = new PeachyAnimator(0.5, this);
	connect(m_animator, SIGNAL(needsRelayout()), this, SLOT(updateLayout()));
	setAcceptsHoverEvents(true);
	m_layout = new PeachyLayout(this);
	setLayout(m_layout);
	connect(TaskManager::TaskManager::self(), SIGNAL(taskAdded(TaskPtr)),
		this, SLOT(addWindowTask(TaskPtr)));
	connect(TaskManager::TaskManager::self(), SIGNAL(taskRemoved(TaskPtr)),
		this, SLOT(removeWindowTask(TaskPtr)));
	reload();
}

void PeachyApplet::addWindowTask(TaskPtr task)
{
	kDebug() << "adding task" << task;
	PeachyIcon* icon = new PeachyIcon(task, this);
	m_layout->add(icon);
	m_animator->add(icon);
	m_tasksHash[task] = icon;
	m_layout->activate();
}

void PeachyApplet::removeWindowTask(TaskPtr task)
{
	kDebug() << "removing task" << task;
	PeachyIcon* icon = m_tasksHash[task];
	if(icon == 0) {
		qWarning("PeachyApplet::removeWindowTask trying to remove non-existant task");
		return;
	}
	m_tasksHash.remove(task);
	m_layout->remove(icon);
	m_animator->remove(icon);
	delete icon;
	m_layout->activate();
}

void PeachyApplet::updateLayout()
{
	m_layout->activate();
}

void PeachyApplet::reload()
{
	m_layout->clear();
	m_animator->clear();
	m_tasksHash.clear();
	
	TaskManager::TaskManager *manager = TaskManager::TaskManager::self();

	TaskManager::TaskDict tasks = manager->tasks();
	QHashIterator<WId, TaskPtr> iter(tasks);

	while(iter.hasNext()) {
		iter.next();
		addWindowTask(iter.value());
	}
}

void PeachyApplet::hoverLeaveEvent(QGraphicsSceneHoverEvent* e)
{
	kDebug() << "hover leaved";
	m_animator->reinit();
	update();
}

#include "peachy_applet.moc"
