/***************************************************************************
 *   Copyright (C) 2009 by Kai Dombrowe <just89@gmx.de>                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

// own
#include "ratingwidget.h"
#include "game.h"

// Qt
#include <QGraphicsWidget>
#include <QPainter>
#include <QGraphicsSceneMouseEvent>

// KDE
#include <Nepomuk/KRatingPainter>


RatingWidget::RatingWidget(QGraphicsWidget *parent, Game *game)
    : QGraphicsWidget(parent),
    m_game(game)
{

    m_ratingPainter = new KRatingPainter;
    m_hoverRating = -1;
    
    connect(m_game, SIGNAL(changed()), this, SLOT(gameChanged()));
    
    setAcceptHoverEvents(true);
    
}


RatingWidget::~RatingWidget()
{

    delete m_ratingPainter;

}


void RatingWidget::gameChanged()
{

    update();

}
    

void RatingWidget::paint(QPainter *painter, const QStyleOptionGraphicsItem *options, QWidget *widget)
{
    
    Q_UNUSED(options);
    Q_UNUSED(widget);
    
    m_ratingPainter->paintRating(painter,
                                 contentsRect().toRect(),
                                 Qt::AlignVCenter|Qt::AlignLeft,
                                 m_game->rating(),
                                 m_hoverRating);
   // painter->drawRect(contentsRect());
    
}
    

QSizeF RatingWidget::sizeHint(Qt::SizeHint which, const QSizeF &constraint) const
{

    Q_UNUSED(which);
    Q_UNUSED(constraint);

    QSizeF hint;
    
    hint.setHeight(KIconLoader::SizeSmall);
    hint.setWidth((KIconLoader::SizeSmall+m_ratingPainter->spacing())*(m_ratingPainter->maxRating()/2));

    switch (which) {
        case Qt::MinimumSize: return hint;
        case Qt::PreferredSize: return hint;
        case Qt::MinimumDescent: return hint;
        case Qt::MaximumSize: return QSizeF(QWIDGETSIZE_MAX, QWIDGETSIZE_MAX);
        case Qt::NSizeHints: return hint;
        default: return hint;
    }
    
}
    
    
void RatingWidget::mousePressEvent(QGraphicsSceneMouseEvent *event)
{

    const int rating = m_ratingPainter->getRatingFromPosition(contentsRect().toRect(),
                                                           Qt::AlignVCenter|Qt::AlignLeft,
                                                           layoutDirection(),
                                                           event->pos().toPoint());
    if (rating != -1) {
        event->accept();
    } else {
        event->ignore();
    }
    
}
    
    
void RatingWidget::hoverMoveEvent(QGraphicsSceneHoverEvent *event)
{

    const int rating = m_ratingPainter->getRatingFromPosition(contentsRect().toRect(),
                                                           Qt::AlignVCenter|Qt::AlignLeft,
                                                           layoutDirection(),
                                                           event->pos().toPoint());
    
    if (m_hoverRating != rating) {
        m_hoverRating = rating;
        update();
    }

}


void RatingWidget::hoverLeaveEvent(QGraphicsSceneHoverEvent *event)
{

    Q_UNUSED(event);
    if (m_hoverRating != -1) {
        m_hoverRating = -1;
        update();
    }
    
}


void RatingWidget::mouseReleaseEvent(QGraphicsSceneMouseEvent *event)
{

    const int rating = m_ratingPainter->getRatingFromPosition(contentsRect().toRect(),
                                                           Qt::AlignVCenter|Qt::AlignLeft,
                                                           layoutDirection(),
                                                           event->pos().toPoint());
    if (rating != -1) {
        m_game->setRating(rating);
        event->accept();
    } else {
        event->ignore();
    }
    
}


void RatingWidget::mouseMoveEvent(QGraphicsSceneMouseEvent *event)
{


    Q_UNUSED(event);

}


void RatingWidget::wheelEvent(QGraphicsSceneWheelEvent *event)
{

    event->ignore();

}



#include "ratingwidget.h"
