#include "webwatcher.h"

#include <QGraphicsLinearLayout>
#include <QGraphicsGridLayout>
#include <QTimer>
#include <QGraphicsDropShadowEffect>

#include <KDebug>

// Plasma
#include <Plasma/WebView>

#include "thumbnailicon.h"
#include "kwebthumbnailer.h"

WebWatcher::WebWatcher(QObject *parent, const QVariantList &args)
    : Plasma::PopupApplet(parent, args),
      m_widget(0)
{
    setAspectRatioMode(Plasma::IgnoreAspectRatio);
    setPopupIcon("internet-web-browser");
    setMinimumSize(400, 600);
}

WebWatcher::~WebWatcher()
{
}

void WebWatcher::init()
{

}

#define THUMBNAIL_COUNT (6)
#define THUMBNAIL_ROW_LENGTH (3)

QGraphicsLayoutItem *WebWatcher::createThumbnailArea( QGraphicsWidget *parent )
{
    QGraphicsGridLayout *grid = new QGraphicsGridLayout;
    grid->setPreferredSize( QSizeF(380, 400) );

    grid->setVerticalSpacing(2);
    grid->setHorizontalSpacing(16);

    for ( int i = 0; i < THUMBNAIL_COUNT; i++ ) {
        m_thumbs[i] = new ThumbnailIcon(parent);
        grid->addItem( m_thumbs[i], i / THUMBNAIL_ROW_LENGTH, i % THUMBNAIL_ROW_LENGTH );
    }

    m_thumbs[0]->setUrl(QUrl("http://www.kde.org/"));
    m_thumbs[1]->setUrl(QUrl("http://news.bbc.co.uk/"));
    m_thumbs[2]->setUrl(QUrl("http://planet.kde.org/"));
    m_thumbs[3]->setUrl(QUrl("http://dot.kde.org/"));
    m_thumbs[4]->setUrl(QUrl("http://xkcd.com/"));
    m_thumbs[5]->setUrl(QUrl("http://labs.trolltech.com/blogs/"));

    // Hack to avoid a big hit on startupx
    int delay = 0;
    for ( int i = 0; i < THUMBNAIL_COUNT; i++ ) {
        QTimer::singleShot( delay, m_thumbs[i], SLOT(start()) );
        delay += 2000;
    }

    return grid;
}

QGraphicsLayoutItem *WebWatcher::createHeadlineArea( QGraphicsWidget *parent )
{
    Plasma::WebView *webview = new Plasma::WebView(parent);
    webview->setHtml( QString(""
        "<html><body>"
        "<h4>World Ends on Thursday</h4>'We're all going to die say world leaders."
        "<h4>'Last Days' Parties Spread</h4>We might as well have a good time say the masses."
        "<h4>Enlightenment 17 Released</h4>Rasterman apologies for the delay."
        "</body></html>")
    );

    return webview;
}

QGraphicsWidget* WebWatcher::graphicsWidget()
{
    if (m_widget)
        return m_widget;

    m_widget = new QGraphicsWidget(this);

    // Outer layout
    QGraphicsLinearLayout *outer = new QGraphicsLinearLayout( Qt::Vertical, m_widget);
    
    m_widget->setLayout(outer);

    // Thumbnail area
    outer->addItem( createThumbnailArea(m_widget) );
    outer->addItem( createHeadlineArea(m_widget) );

    return m_widget;
}

#include "webwatcher.moc"
