/*
# * Copyright(c) 2008 Matthew Adams <roguestar191 at comcast.net>
# *
# * This program is free software; you can redistribute it and/or
# * modify it under the terms of the GNU General Public License as
# * published by the Free Software Foundation; either version 2 of
# * the License, or(at your option) any later version.
# *
# * This program is distributed in the hope that it will be useful,
# * but WITHOUT ANY WARRANTY; without even the implied warranty of
# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# * GNU General Public License for more details.
# *
# * You should have received a copy of the GNU General Public License
# * along with this program.  If not, see <http//www.gnu.org/licenses/>.
# */
#ifndef UI_PLASMA_TOOLS_CPP
#define UI_PLASMA_TOOLS_CPP

#include "../includes/Ui_PlasmaTools.h"
#include "../includes/Ui_Dialog.h"

#include <kiconloader.h>
#include <kstandarddirs.h>
#include "../welcome.h"

#include <iostream>
#include <fcntl.h>

namespace welcome_plasmoid {

	ScrollableListWidget::ScrollableListWidget(QGraphicsItem *top) : QGraphicsProxyWidget(top), ScrollArea(new QScrollArea()), ScrollWidget(new QWidget()), ListWidget(new QListWidget(ScrollWidget)), Layout(new QGraphicsLinearLayout()), ScrollLayout(new QVBoxLayout(ScrollWidget)){
		ScrollArea->setWidget(ScrollWidget);
		Layout->addItem(this);		
		this->setWidget(ScrollArea);
		ScrollLayout->addWidget(ListWidget);
	}
	class Ui_WelcomePlasmoid;
	aPid::aFp::aFp(FILE*_fp, aPid* _top) {
		fp = _fp;
		count = 1;
		top = _top;

	}
	aPid::aFp::aFp(const aFp& old) {
		top = old.top;
		fp = old.fp;
		count = 1;
		count += old.count;
	}
	aPid::aFp::~aFp() {
		std::vector<aFp>::iterator iter = top->files.begin(), end = top->files.end();
		bool found = false;
		for(;iter != end;iter++) 
			if( (*(iter)).fp == fp ) {
				(*(iter)).count--;
				found = true;
			}
		if(count == 0 && found == false) {
			pclose(fp);
		}
	}
	void aPid::addFile(FILE*fp) {
		bool found = false;
		std::vector<aFp>::iterator iter = files.begin(), end = files.end();
		for(;iter != end;iter++) {
			if((*(iter)).fp == fp) {
				found = true;
				(*(iter)).count++;
			}
		}
		if(found == false) {
			aFp nfp = aFp(fp, this);
			files.push_back(nfp);
		}
	}
	void aPid::remFile(FILE*fp) {
		std::vector<aFp>::iterator iter = files.begin(), end = files.end();
		for(;iter != end;iter++) {
			if((*(iter)).fp == fp) {
				(*(iter)).count--;
				if((*(iter)).count == 0)
					files.erase(iter);
				return;
			}
		}
	}
	ssize_t aPid::read() {
		char buf[10000];
		memset(buf, 0, sizeof(buf));
		if(feof(fp) == true) alive = false;
		ssize_t ret = fread(buf, sizeof(buf)-1, 1, fp);
		buffer.append(buf);
		return ret;
	}
	aPid::~aPid() {
		remFile(fp);
	}
	aPid::aPid(FILE* _fp) { 
		fp = _fp; 
		addFile(fp);
		alive = true;
	}
	aPid::aPid(const aPid &old) {
		buffer = old.buffer;
		fp = old.fp;
		addFile(fp);
		alive = old.alive;
	}
	QString runCommand(QString cmd) {
		aPid pid = aPid(popen(cmd.toLocal8Bit().data(), "r"));
		while(pid.alive)
			pid.read();
		return pid.buffer;
	}

	Dialog_Tools::Dialog_Tools(){}
	Dialog_Tools::~Dialog_Tools(){}

	void Dialog_Tools::invalidateLayouts(){}


	Url_Proxy::Url_Proxy(Ui_WelcomePlasmoid*_top) : QGraphicsProxyWidget(NULL), top(_top) 
	{
		url = new KHistoryComboBox();
		setWidget(url);
		url->setAttribute(Qt::WA_NoSystemBackground);
		setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
		url->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
		connect(url, SIGNAL(textChanged(QString)), this, SIGNAL(textChanged(QString)));
		connect(url, SIGNAL(returnPressed(QString)), this, SIGNAL(returnPressed(QString)));
	}
	TransparentFrame::TransparentFrame(Ui_WelcomePlasmoid* parent) : QGraphicsWidget(parent) {
		top = parent;
		this->show();
		this->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);

	}

	ResizeFrame::ResizeFrame(Ui_WelcomePlasmoid*parent) : TransparentFrame(parent) {
		gridlayout = NULL;
		linearlayout = NULL;
		top = parent;
	}
	void ResizeFrame::setLayout(QGraphicsGridLayout*l) {
		gridlayout = l;
		l->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
	}
	void ResizeFrame::setLayout(QGraphicsLinearLayout*l) {
		linearlayout = l;
		l->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
	}
	void ResizeFrame::updateGeometry() {
		QGraphicsWidget::updateGeometry();
//		resizeEvent(NULL);
	}
	void ResizeFrame::resizeEvent(QGraphicsSceneResizeEvent*e) {
		if(e != NULL) 
			QGraphicsWidget::resizeEvent(e);	
		int diff = top->top->size().width()-675;
		diff = (diff/4);
		if(linearlayout != NULL) {
			linearlayout->setSpacing(15+diff);
//			linearlayout->invalidate();
//			linearlayout->activate();
		} else if(gridlayout != NULL) {
			gridlayout->setSpacing(15+diff);
//			gridlayout->invalidate();
//			gridlayout->activate();
		}
//		e->accept();
	}
	WebButton::WebButton(Ui_WelcomePlasmoid *top) : Plasma::PushButton(top) {		
	}
	void WebButton::setLabelText(QString &s) {
		setText(s);
	}
	MyQtLabel::MyQtLabel(int, QWidget *par) : QLabel(par), point(0,0)  { }
	MyQtLabel::MyQtLabel(GloweyLabel*par, Ui_WelcomePlasmoid* _top, QWidget *Qpar) : QLabel(Qpar), point(0,0)  {
		setup (par, _top);
	}
	void MyQtLabel::setup(GloweyLabel* par, Ui_WelcomePlasmoid* _top) {
		top = _top;
		par = par;
		currentLink = "";

		_top->GlowAnimation = _top->top->config().readEntry("Use Glow Animation", false);
		_top->FadeAnimation = _top->top->config().readEntry("Use Fade Animation", true);
		
		
		setAutoFillBackground(true);
		setWordWrap(true);
		setStyleSheet("");
		par->setStyleSheet("");
		connect(this, SIGNAL(linkHovered(QString)), this, SLOT(linkHovered(QString)));
		connect(this, SIGNAL(clickedOn(QString)), par, SLOT(linkClicked()));
// Change this later..
		connect(this, SIGNAL(clickedOn(QString)), par, SIGNAL(linkActivated(QString)));

		InsideWidget = false;
		brightness = 0;
		waveup = true;
		wave = 0.0f;
		top->factor = _top->top->config().readEntry("Opacity Factor", 10.0f);
		top->maxRise = _top->top->config().readEntry("Max Rise", 25.0f);
		top->maxWave = _top->top->config().readEntry("Max Wave", 30.0f);
		wavetop = top->maxRise + top->maxWave + 5.0f;
		
		backgroundImage = &top->backgroundImage;
		backgroundImage->setElementPrefix(_top->top->config().readEntry("Background Image Element", QString("hover")));
		hasPixmap = false;
		animating = false;
		painter = new QPainter();
		mytimer = new QTimer();

		QObject::connect(mytimer, SIGNAL(timeout()), this, SLOT(animate()));
	}
	void MyQtLabel::linkHovered(QString link) {
		if(link != "") {
			currentLink = link;
			InsideWidget = true;
			if(animating == false) {
				startAnimate();
			}
		}
	}
	void MyQtLabel::mousePressEvent(QMouseEvent*e) {
		QLabel::mousePressEvent(e);
		if(InsideWidget) {
			emit(clickedOn(""));
			emit(clickedOn(currentLink));
		}
	}
	void MyQtLabel::generatePalette() {
		QColor mycolor = Plasma::Theme::defaultTheme()->color(Plasma::Theme::TextColor);
		setFont(Plasma::Theme::defaultTheme()->font(Plasma::Theme::DefaultFont));
		QPalette p = palette();
		p.setColor(QPalette::Normal, QPalette::WindowText, mycolor);
		p.setColor(QPalette::Inactive, QPalette::WindowText, mycolor);
		setPalette(p);

	}
	void MyQtLabel::resizeEvent(QResizeEvent*e) {
		if(!top->EXITING) {
			backgroundImage->resizeFrame(size());
			backgroundImage->setElementPrefix("hover");
			mypixmap = backgroundImage->framePixmap().toImage();
			QLabel::resizeEvent(e);
			hasPixmap = true;
		}
	}
	void MyQtLabel::paintEvent(QPaintEvent*e) {
		if(hasPixmap == false) {
			QLabel::paintEvent(e);
			return;
		}
		painter->begin(this);
		painter->setOpacity((wave+brightness)/wavetop);
		if(painter->opacity() != 0) {
			painter->drawImage(point, mypixmap);
		}
		painter->end();
		QLabel::paintEvent(e);
	}
	void MyQtLabel::leaveEvent(QEvent*e) {
		InsideWidget = false;
		if(top->FadeAnimation) mytimer->start(1000/32);
		QLabel::leaveEvent(e);
	}
	void MyQtLabel::startAnimate() {
		mytimer->start(1000/32);
		animating = true;
	}
	void MyQtLabel::animate() {
		if(!isVisible()) InsideWidget = false;
		if(InsideWidget) {
			if(brightness < top->maxRise && wave == 0) {
				brightness += top->factor;
			} else {
				if(waveup) {
					wave += top->factor;
					if(wave >= top->maxWave) {
						waveup = false;
					}
				} else {
					if(top->FadeAnimation) {
						mytimer->stop();
					}
					wave -= top->factor;
					if(wave <= 0) {
						waveup = true;
					}
				}
			}
		} else {
			if(wave > 0) {
				wave -= top->factor;
			} else if(brightness > 0) {
				brightness -= top->factor;
			} else {
				wave = 0;
				waveup = true;
				mytimer->stop();
				animating = false;
			}
		}
		repaint();
	}
	GloweyLabel::GloweyLabel() : Plasma::Label(NULL), icon(NULL)
	{ 
		isSetup = false;
	}
	GloweyLabel::GloweyLabel(Ui_WelcomePlasmoid*top, bool setMax) : Plasma::Label(NULL), icon(NULL) {
		isSetup = true;
		setup(top, setMax);
	}
	void GloweyLabel::setup(Ui_WelcomePlasmoid*top, bool setMax, bool useScroll) {
                
		if(!useScroll) {
			labelWidget = new MyQtLabel(1, NULL);
			labelWidget->setup(this, top);
			labelWidget->setAttribute(Qt::WA_NoSystemBackground, true);
		}
		iconpath = KIconLoader::global()->iconPath("terminal", -128);
		iconpath = iconpath.left(iconpath.indexOf("128x128"));
		filepath = KStandardDirs::locate("data", "welcome/about.html").replace("about.html", "");

		this->setAttribute(Qt::WA_NoSystemBackground, true);
		if(!useScroll) {
			scroll = false;
			setWidget(labelWidget);
		} else {
			scroll = true;
			QWidget *scrollAreaWidgetContents;
			QVBoxLayout *verticalLayout;
			scrollArea = new QScrollArea();
			scrollArea->setWidgetResizable(true);
			scrollArea->setFrameShape(QFrame::NoFrame);
			scrollArea->setFrameShadow(QFrame::Plain);
			scrollAreaWidgetContents = new QWidget();
			verticalLayout = new QVBoxLayout(scrollAreaWidgetContents);
			labelWidget = new MyQtLabel(1, scrollAreaWidgetContents);
			labelWidget->setup(this, top);
			labelWidget->setAttribute(Qt::WA_NoSystemBackground, true);
			verticalLayout->addWidget(labelWidget);
			scrollArea->setWidget(scrollAreaWidgetContents);
			scrollArea->setAttribute(Qt::WA_NoSystemBackground, true);
			scrollAreaWidgetContents->setAttribute(Qt::WA_NoSystemBackground, true);
			scrollAreaWidgetContents->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
			setWidget(scrollArea);
		}

		labelWidget->generatePalette();
//		scrollArea->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);	

//		this->setMinimumSize(140,120);
		if(!setMax) 
			this->setMaximumSize(200,200);
		 else 
			this->setMaximumSize(167722,167722);
	
		connect(this, SIGNAL(linkActivated(QString)), top, SLOT(setCurrentUrl(QString)));
	}
	void GloweyLabel::setMinimumSize(int x, int y) {
		Plasma::Label::setMinimumSize(x,y);
		labelWidget->setMinimumSize(x,y);
		if(scroll)scrollArea->setMinimumSize(x,y);
	}
	void GloweyLabel::setMaximumSize(int x, int y) {
		Plasma::Label::setMaximumSize(x,y);
		labelWidget->setMaximumSize(x,y);
		if(scroll)scrollArea->setMaximumSize(x,y);


	}
	void GloweyLabel::setLabelText(QString text) {
		//KDebug << "Depricated call to setLabelText in GloweyLabel.";
	//	return;
		QString color = KGlobalSettings::activeTextColor().name();
		text = text.replace("a href=", "a style='text-decoration: none;color:" + color + ";' href=");
		text = text.replace("<%icon>", iconpath);
		text = text.replace("<%username>", runCommand("whoami"));
		text = text.replace("<%file>", filepath);
		labelWidget->setText(text);
	}
	void GloweyLabel::linkHovered() {}


	void GloweyLabel::paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget) {
		Plasma::Label::paint(painter, option, widget);


/*
		if(text ==NULL|| icon == NULL|| link == NULL) return;
		// Icon takes top 60% text takes lower 40% draw text 8px from icon and center it.
		// Draw launcher name at 20%, then description at 20%
		icon->resize(size().width(), size().height()
		
		QFontMetrics fm(smallFont);
		smallFont.setPixelSize(qMax(contentsRect.height()/10, fm.ascent()));
*/
	}
	void GloweyLabel::setIcon(QString icon) {

//		if(icon != NULL) delete icon;
//		icon = new Plasma::Svg(labelWidget->top);
//		icon->setImagePath(icon);
	}
	void GloweyLabel::setDescription(QString in) {
//		text = in;
	}
	void GloweyLabel::setLauncher(QString command) {
		launcher = command;	
	}
        void GloweyLabel::setName(QString in) {
//              text = in;
        }
        void GloweyLabel::linkClicked() {
//              text = in;
        }
	GloweyIcon::GloweyIcon(Ui_WelcomePlasmoid* _top) : Plasma::IconWidget(NULL), top(_top) {
                InsideWidget = false;
                brightness = 0;
                waveup = true;
                wave = 0.0f;
                top->factor = _top->top->config().readEntry("Opacity Factor", 10.0f);
                top->maxRise = _top->top->config().readEntry("Max Rise", 25.0f);
                top->maxWave = _top->top->config().readEntry("Max Wave", 30.0f);
                wavetop = top->maxRise + top->maxWave + 5.0f;
		activated = false;
		postactivated = false;
                backgroundImage = &top->backgroundImage;
                backgroundImage->setElementPrefix(_top->top->config().readEntry("Background Image Element", QString("hover")));
                hasPixmap = false;
                animating = false;
                mytimer = new QTimer();
		t = new QTimer();
		connect(t, SIGNAL(timeout()), this, SLOT(postActivate()));
                connect(mytimer, SIGNAL(timeout()), this, SLOT(animate()));
		connect(this, SIGNAL(clicked()), this, SLOT(linkClicked()));
		connect(this, SIGNAL(linkActivated(QString)), top, SLOT(setCurrentUrl(QString)));
	}
	void GloweyIcon::postActivate() {
		t->stop();
		if(postactivated) {
			postactivated = false;
		}
		if(activated) {
			postactivated = true;
			activated = false;
		}

		update();
	}
	void GloweyIcon::linkClicked() {
		if(link == "") return;
		emit linkActivated("");
		emit linkActivated(link);
		activated = true;
		update();
	}
	void GloweyIcon::paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget) {
		Plasma::IconWidget::paint(painter, option, widget);
                if(hasPixmap == false) {
                        return;
                }
		if(!activated && !postactivated)
	                painter->setOpacity((wave+brightness)/wavetop);
		if(postactivated) {
			painter->setOpacity(50);
			t->start(300);
		}
		if(activated) {
			painter->setOpacity(0);
			t->start(300);
		}
                if(painter->opacity() != 0) {
                        painter->drawImage(point, mypixmap);
                }
	}
	void GloweyIcon::hoverEnterEvent(QGraphicsSceneHoverEvent*e) {
		Plasma::IconWidget::hoverEnterEvent(e);
		InsideWidget = true;
		if(animating == false) {
			mytimer->start(1000/32);
			animating = true;
		}
	}
	void GloweyIcon::hoverLeaveEvent(QGraphicsSceneHoverEvent*e) {
		Plasma::IconWidget::hoverLeaveEvent(e);
		InsideWidget = false;
		if(top->FadeAnimation) mytimer->start(1000/32);
	}
	void GloweyIcon::resizeEvent ( QGraphicsSceneResizeEvent * event ) {
		if(!top->EXITING) {
			Plasma::IconWidget::resizeEvent(event);
	                backgroundImage->resizeFrame(size());
	                backgroundImage->setElementPrefix("hover");
	                mypixmap = backgroundImage->framePixmap().toImage();
	                hasPixmap = true;
		}
	}
	void GloweyIcon::setLink(QString text) {
                text = text.replace("<%username>", runCommand("whoami"));
		text = text.replace("<%file>", KStandardDirs::locate("data", "welcome/about.html").replace("about.html", ""));
		link = text;
	}
	void GloweyIcon::animate() {
                if(!isVisible()) InsideWidget = false;
                if(InsideWidget) {
                        if(brightness < top->maxRise && wave == 0) {
                                brightness += top->factor;
                        } else {
                                if(waveup) {
                                        wave += top->factor;
                                        if(wave >= top->maxWave) {
                                                waveup = false;
                                        }
                                } else {
                                        if(top->FadeAnimation) {
                                                mytimer->stop();
                                        }
                                        wave -= top->factor;
                                        if(wave <= 0) {
                                                waveup = true;
                                        }
                                }
                        }
                } else {
                        if(wave > 0) {
                                wave -= top->factor;
                        } else if(brightness > 0) {
                                brightness -= top->factor;
                        } else {
                                wave = 0;
                                waveup = true;
                                mytimer->stop();
                                animating = false;
                        }
                }
                update();		
	}

}

#include "../includes/Ui_PlasmaTools.moc"
#endif
