/*
# * Copyright(c) 2008 Matthew Adams <roguestar191 at comcast.net>
# *
# * This program is free software; you can redistribute it and/or
# * modify it under the terms of the GNU General Public License as
# * published by the Free Software Foundation; either version 2 of
# * the License, or(at your option) any later version.
# *
# * This program is distributed in the hope that it will be useful,
# * but WITHOUT ANY WARRANTY; without even the implied warranty of
# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# * GNU General Public License for more details.
# *
# * You should have received a copy of the GNU General Public License
# * along with this program.  If not, see <http//www.gnu.org/licenses/>.
# *
# * Note: right now this is a copy of a test plasmoid in playground...
# */

#include "../welcome.h"
#include <iostream>
#include <QPainter>
#include <QFontMetrics>
#include <QSizeF>
 
#include <plasma/svg.h>
#include <plasma/theme.h>

#include <kstandarddirs.h>
#include <../includes/Ui_BackgroundWidget.h>

PlasmaWelcome::PlasmaWelcome(QObject *parent, const QVariantList &args)
    : Plasma::PopupApplet(parent, args), openDialog(new KOpenWithDialog(NULL)), selectedButton(NULL),
    graphics_widget(new welcome_plasmoid::BackgroundProxy(this)),
    m_svg(graphics_widget),
    m_icon("document")
	
{
	openDialog->hide();
	m_svg.setImagePath("welcome/kde_watermark");
      connect(openDialog, SIGNAL(accepted()), this, SLOT(selectLauncherAccepted()));
    // this will get us the standard applet background, for free!
//    setBackgroundHints(DefaultBackground);
	resize(675, 463);
	setHasConfigurationInterface(true);
	KConfigGroup cg = config();
	setTipDB(cg.readEntry("Default Tipfile", KStandardDirs::locate("data", "welcome/tips")));
	graphics_widget->resize(674,463);
	priv = new welcome_plasmoid::Ui_WelcomePlasmoid(this);
	setAspectRatioMode(Plasma::IgnoreAspectRatio);
	graphics_widget->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding );
	setPopupIcon("konqueror");
	autoPopup = cg.readEntry("Show on Startup", true);	
//	addAssociatedWidget(background_widget);

}
 
 
PlasmaWelcome::~PlasmaWelcome()
{
    if (hasFailedToLaunch()) {
        // Do some cleanup here
    } else {
        // Save settings
    }
	delete priv;
}
 #include <signal.h>

void PlasmaWelcome::createConfigurationInterface(KConfigDialog *parent) {
	Plasma::PopupApplet::createConfigurationInterface(parent);
    QWidget *widget = new QWidget();
    ui.setupUi(widget);
    parent->addPage(widget, i18n("General"), icon());
    connect(parent, SIGNAL(accepted()), SLOT(configAccepted()));
    ((welcome_plasmoid::Ui_WelcomePlasmoid*)priv)->fillTipFileBox(ui.tipFileBox);
    KConfigGroup cg = config();

  ui.defaultWebsite->setText(cg.readEntry("Default Website", "http://www.kde.org/"));
  ui.tipFileBox->setCurrentIndex(ui.tipFileBox->findText(cg.readEntry("Default Tipfile", KStandardDirs::locate("data", "welcome/tips")))); 
  ui.defaultDownloadFolder->setPath(cg.readEntry("Default Download Folder", "~/"));
  ui.defaultDownloadFolder->setMode(KFile::Directory);

  ui.useGlowAnimation->setChecked(cg.readEntry("Use Glow Animation", false));
  ui.useFadeAnimation->setChecked(cg.readEntry("Use Fade Animation", true));
  ui.useTabOnWelcome->setChecked(cg.readEntry("Use Tab On Welcome", false));
  ui.useTabOnBrowser->setChecked(cg.readEntry("Use Tab On Browser", true));
  ui.useTabOnLinks->setChecked(cg.readEntry("Use Tab On Links", true));
  ui.changeTabAnimation->setChecked(cg.readEntry("Change Tab Animation", true));
  ui.showOnStartup->setChecked(cg.readEntry("Show on Startup", true));

  ui.defaultTab->setValue( cg.readEntry("Default Tab", 0) + 1);
  ui.maxRise->setValue( cg.readEntry("Max Rise", 25.0));
  ui.maxWave->setValue( cg.readEntry("Max Wave", 30.0));
  ui.opacityFactor->setValue( cg.readEntry("Opacity Factor", 10));

  ui.commandLauncher1->setText( cg.readEntry("Command Name 1", "konsole"));
  ui.commandLauncher2->setText( cg.readEntry("Command Name 2", "ksystemlog"));
  ui.commandLauncher3->setText( cg.readEntry("Command Name 3", "systemsettings"));
  ui.commandLauncher4->setText( cg.readEntry("Command Name 4", "kpackage"));

  ui.defaultLauncher->setText( cg.readEntry("Default Applet", "launcher"));
  ui.embedPrefY->setValue(cg.readEntry("Embed Pref Y", 300));

  connect(ui.commandLauncher1, SIGNAL(clicked()), this, SLOT(openLauncherDialog1() ));
  connect(ui.commandLauncher2, SIGNAL(clicked()), this, SLOT(openLauncherDialog2() ));
  connect(ui.commandLauncher3, SIGNAL(clicked()), this, SLOT(openLauncherDialog3() ));
  connect(ui.commandLauncher4, SIGNAL(clicked()), this, SLOT(openLauncherDialog4() ));


/*    ui.showDate->setChecked(m_showDate);
    ui.showYear->setChecked(m_showYear);
    ui.showDay->setChecked(m_showDay);
    ui.secondsCheckbox->setChecked(m_showSeconds);
    ui.showTimeZone->setChecked(m_showTimezone);
    ui.plainClockFontBold->setChecked(m_plainClockFont.bold());
    ui.plainClockFontItalic->setChecked(m_plainClockFont.italic());
    ui.plainClockFont->setCurrentFont(m_plainClockFont);
    ui.useCustomColor->setChecked(m_useCustomColor);
    ui.plainClockColor->setColor(m_plainClockColor);
*/
}
void PlasmaWelcome::configAccepted() {
//	Plasma::PopupApplet::configAccepted();
	std::cout << "Config accepted, hoozah!\n";
    KConfigGroup cg = config();

   QString defaultWebsite = ui.defaultWebsite->text();
   if(!defaultWebsite.contains(":")) defaultWebsite.insert(0, "http://");
   cg.writeEntry("Default Website", defaultWebsite);
   cg.writeEntry("Use Glow Animation", ui.useGlowAnimation->isChecked());
   cg.writeEntry("Use Fade Animation", ui.useFadeAnimation->isChecked());
   if(ui.tipFileBox->currentText() != "" )
	cg.writeEntry("Default Tipfile", ui.tipFileBox->currentText());
   else 
	cg.writeEntry("Default Tipfile", KStandardDirs::locate("data", "welcome/tips"));
   cg.writeEntry("Default Download Folder", ((QUrl)(ui.defaultDownloadFolder->url())).toString().append("/"));
   cg.writeEntry("Default Tab", ui.defaultTab->value()-1);
   cg.writeEntry("Max Rise", ui.maxRise->value());
   cg.writeEntry("Max Wave", ui.maxWave->value());
   cg.writeEntry("Opacity Factor", ui.opacityFactor->value());
   cg.writeEntry("Use Tab On Welcome", ui.useTabOnWelcome->isChecked());
   cg.writeEntry("Use Tab On Browser", ui.useTabOnBrowser->isChecked());
   cg.writeEntry("Use Tab On Links", ui.useTabOnLinks->isChecked());
   cg.writeEntry("Change Tab Animation", ui.changeTabAnimation->isChecked());
   cg.writeEntry("Show on Startup", ui.showOnStartup->isChecked());
// Note: DONT add any more config on the fly like this, instead, hook to configChanged() or configNeedsSaving().. it's a lot easier
   ((welcome_plasmoid::Ui_WelcomePlasmoid*)priv)->tabChanged(((welcome_plasmoid::Ui_WelcomePlasmoid*)priv)->tabWidget->currentIndex());
   ((welcome_plasmoid::Ui_WelcomePlasmoid*)priv)->factor =  ui.opacityFactor->value();
   ((welcome_plasmoid::Ui_WelcomePlasmoid*)priv)->maxRise = ui.maxWave->value();
   ((welcome_plasmoid::Ui_WelcomePlasmoid*)priv)->maxWave = ui.maxWave->value();
   ((welcome_plasmoid::Ui_WelcomePlasmoid*)priv)->GlowAnimation = ui.useGlowAnimation->isChecked();
   ((welcome_plasmoid::Ui_WelcomePlasmoid*)priv)->FadeAnimation = ui.useFadeAnimation->isChecked();

    cg.writeEntry("Default Applet", ui.defaultLauncher->text()); 
    cg.writeEntry("Embed Pref Y", ui.embedPrefY->value());

  ui.defaultLauncher->setText( cg.readEntry("Default Applet", "launcher"));
  ui.embedPrefY->setValue(cg.readEntry("Embed Pref Y", 300));

   setTipDB(ui.tipFileBox->currentText());
   update();
    emit sizeHintChanged(Qt::PreferredSize);
    emit configNeedsSaving();
    emit configChanged();
}
void PlasmaWelcome::saveConfig() {
	emit configNeedsSaving();
}
QGraphicsWidget* PlasmaWelcome::graphicsWidget() {
	return graphics_widget;
}
//void PlasmaWelcome::configChanged();
void PlasmaWelcome::init()
{
 
    // A small demonstration of the setFailedToLaunch function
    if (m_icon.isNull()) {
        setFailedToLaunch(true, "No world to say hello");
    }
} 
void PlasmaWelcome::setTipDB(QString fname) {
	tipdb.load(fname);
}
QString PlasmaWelcome::getTip() {
	return tipdb.tip();
}
QString PlasmaWelcome::nextTip() {
	tipdb.nextTip();
	return tipdb.tip();
}
QString PlasmaWelcome::prevTip() {
	tipdb.nextTip();
	return tipdb.tip();
}
bool PlasmaWelcome::isHtml() {
	return tipdb.isHtml();
}
KConfigGroup PlasmaWelcome::getConfig() {
	return config();
}
void PlasmaWelcome::selectLauncherAccepted() {

	KService::Ptr ptr =  openDialog->service();

	std::cout << "The launcher was: " << " " << ptr->exec().toLocal8Bit().data() << " Library: " << ptr->library().toLocal8Bit().data() << " Icon:" << ptr->icon().toLocal8Bit().data() << " comment: " << ptr->comment().toLocal8Bit().data() <<std::endl;
	if(selectedButton == NULL)  { 
		std::cout << "Selected button was null!\n";
		return;
	}
	QString num;
	if(selectedButton == ui.commandLauncher1) {
		num = "1";
	} else if(selectedButton == ui.commandLauncher2) {
		num = "2";
	} else if(selectedButton == ui.commandLauncher3) {
		num = "3";
	} else if(selectedButton == ui.commandLauncher4) {
		num = "4";
	} 
	config().writeEntry(QString("Command Exec ").append(num), ptr->exec() );
	config().writeEntry(QString("Command Desc ").append(num), ptr->comment() );
	QString name = "Error";
	if(ptr->genericName() != "") 
		name = ptr->genericName();
	else
		name =  ptr->name();
	config().writeEntry(QString("Command Name ").append(num), name);
	config().writeEntry(QString("Command Icon ").append(num), ptr->icon() );

        if(selectedButton == ui.commandLauncher1) {
		ui.commandLauncher1->setText(name);
		ui.commandLauncher1->update();
        } else if(selectedButton == ui.commandLauncher2) {
		ui.commandLauncher2->setText(name);
		ui.commandLauncher2->update();
        } else if(selectedButton == ui.commandLauncher3) {
		ui.commandLauncher3->setText(name);
		ui.commandLauncher3->update();
        } else if(selectedButton == ui.commandLauncher4) {
		ui.commandLauncher4->setText(name);
		ui.commandLauncher4->update();
        }

	selectedButton = NULL;
	
}
void PlasmaWelcome::openLauncherDialog1() {
	selectedButton = ui.commandLauncher1;	
	openDialog->show();
}
void PlasmaWelcome::openLauncherDialog2() {
	selectedButton = ui.commandLauncher2;	
	openDialog->show();
}
void PlasmaWelcome::openLauncherDialog3() {
	selectedButton = ui.commandLauncher3;	
	openDialog->show();
}
void PlasmaWelcome::openLauncherDialog4() {
	selectedButton = ui.commandLauncher4;	
	openDialog->show();
}
void PlasmaWelcome::paintInterface(QPainter *p,
        const QStyleOptionGraphicsItem *option, const QRect &contentsRect)
{
	if(autoPopup) {
		static bool ranShowPopup = false;
		if(!ranShowPopup && !isPopupShowing()) {
			showPopup();
			ranShowPopup = true;
		}
	}
//   p->setRenderHint(QPainter::SmoothPixmapTransform);
//   p->setRenderHint(QPainter::Antialiasing);
	Plasma::PopupApplet::paintInterface(p, option, contentsRect);
//	int lheight = graphics_widget->size().height()/4;
//	int diff = lheight-m_svg.size().height();
//	int lwidth = m_svg.size().width()+diff;

//	m_svg.resize(lheight, lwidth);
//	int posx = graphics_widget->size().width()/2-m_svg.size().width()/2;
//	int posy = graphics_widget->size().height()/2-m_svg.size().height()/2;
//	m_svg.paint(p, posx, posy);

    // We place the icon and text
//    p->drawPixmap(7, 0, m_icon.pixmap((int)contentsRect.width(),(int)contentsRect.width()-14));
//    p->save();
//    p->setPen(Qt::white);
//    p->drawText(contentsRect,
//                Qt::AlignBottom | Qt::AlignHCenter,
//                "Hello Plasmoid!");
//    p->restore();
}
 



// This is the command that links your applet to the .desktop file
K_EXPORT_PLASMA_APPLET(welcome, PlasmaWelcome)

#include "../welcome.moc"
