// Cluttered Desktop code starts here
// Edit cluttereddesktop.cpp (not desktop.cpp) and run update.sh

#include <typeinfo>
#include <QTextDocument>
#include <QAbstractTextDocumentLayout>
#include <QGraphicsLinearLayout>
#include <KConfigDialog>
#include <Plasma/Wallpaper>
#include <Plasma/WebView>
#include "ui_config.h"

class ClutteredDesktopPrivate
{
public:
    QGraphicsItem* toolbox;
    QTextDocument doc;
    QGraphicsLinearLayout* layout;
    Plasma::WebView* html;
    bool cashew;
    bool debug;
    Ui::Config ui;
};

void ClutteredDesktop::destroyed()
{
    delete d;
}

void ClutteredDesktop::init()
{
    Containment::init();
    d = new ClutteredDesktopPrivate;

    KConfigGroup cfg = config();
    d->cashew = cfg.readEntry("cashew", true);
    d->debug = cfg.readEntry("debug", false);

    findToolbox();
    if (d->toolbox && !d->cashew) {
        d->toolbox->hide();
    }
    connect(this, SIGNAL(appletAdded(Plasma::Applet*, const QPointF)),
            this, SLOT(appletAddedSlot(Plasma::Applet*, const QPointF)));
    connect(this, SIGNAL(appletRemoved(Plasma::Applet*)), this, SLOT(updateText()));
    connect(this, SIGNAL(screenChanged(int, int, Plasma::Containment*)), this, SLOT(updateText()));
    connect(this, SIGNAL(geometryChanged()), this, SLOT(updateText()));
    /*
    m_settingsAction = new QAction(i18n("Cluttered Desktop Extra Settings"), this);
    m_settingsAction->setIcon(KIcon("configure"));
    connect(m_settingsAction, SIGNAL(triggered(bool)),
            this, SLOT(showAppletConfigurationInterface()));
    */
    QTimer::singleShot(250, this, SLOT(updateText()));
}

void ClutteredDesktop::appletAddedSlot(Plasma::Applet* applet, const QPointF pos)
{
    Q_UNUSED(pos)
    connect(applet, SIGNAL(geometryChanged()), this, SLOT(updateText()));
    updateText();
}

void ClutteredDesktop::updateText()
{
    if (d->debug) {
        QString toolbox = (d->toolbox) ? (d->cashew) ? i18n("Visible") : i18n("Hidden") : i18n("Not Found");
        QString row("<tr><td>%1:</td><td>%2</td></tr>");
        QString text =
                "<h2>Cluttered Desktop</h2><table>" +
                QString(row).arg(i18n("ID")).arg(id()) +
                QString(row).arg(i18n("Screen")).arg(screen()) +
                QString(row).arg(i18n("Geometry")).arg(QString("%1,%2 %3,%4")
                        .arg(geometry().left()).arg(geometry().top())
                        .arg(geometry().width()).arg(geometry().height())) +
                QString(row).arg(i18n("Wallpaper"))
                        .arg((wallpaper()) ? wallpaper()->pluginName() : QString()) +
                QString(row).arg(i18n("Wallpaper mode")).arg((wallpaper()) ?
                        wallpaper()->renderingMode().name() : QString()) +
                QString(row).arg(i18n("Toolbox")).arg(toolbox) +
                "</table><h3>Applets</h3><table>";
        foreach (Applet* applet, applets()) {
            text += QString("<tr><td>%1:</td><td>%2</td><td>%3</td></tr>")
                    .arg(applet->name()).arg(applet->id())
                    .arg(QString("%2,%3 %4,%5")
                        .arg(applet->geometry().left()).arg(applet->geometry().top())
                        .arg(applet->geometry().width()).arg(applet->geometry().height()));
        }
        text += "</table>";
        d->doc.setHtml(text);
        update();
    }
}

void ClutteredDesktop::findToolbox()
{
    d->toolbox = 0;
    foreach (QGraphicsItem* item, childItems()) {
        QString s = typeid(*item).name();
        if (s.contains("DesktopToolBox")) {
            d->toolbox = item;
            break;
        }
    }
}

void ClutteredDesktop::paintInterface(QPainter* painter,
                                     const QStyleOptionGraphicsItem* option,
                                     const QRect& contentsRect)
{
    Q_UNUSED(option)
    Q_UNUSED(contentsRect)
    if (d->debug) {
        QAbstractTextDocumentLayout* layout = d->doc.documentLayout();
        QAbstractTextDocumentLayout::PaintContext context;

        painter->save();
        painter->translate(52, 52);
        context.palette.setColor(QPalette::Text, QColor(Qt::black));
        layout->draw(painter, context);
        painter->translate(-2, -2);
        context.palette.setColor(QPalette::Text, QColor(Qt::white));
        layout->draw(painter, context);
        painter->restore();
    }
}

void ClutteredDesktop::createConfigurationInterface(KConfigDialog* parent)
{
    QWidget* widget = new QWidget;
    d->ui.setupUi(widget);

    d->ui.cashewCheckBox->setCheckState((Qt::CheckState)((int)d->cashew * 2));
    d->ui.debugCheckBox->setCheckState((Qt::CheckState)((int)d->debug * 2));

    parent->setButtons(KDialog::Ok | KDialog::Cancel | KDialog::Apply);
    parent->addPage(widget, i18n("Desktop"), "user-desktop");
    connect(parent, SIGNAL(applyClicked()), this, SLOT(configAccepted()));
    connect(parent, SIGNAL(okClicked()), this, SLOT(configAccepted()));
}

void ClutteredDesktop::configAccepted()
{
    KConfigGroup cfg = config();

    d->cashew = (d->ui.cashewCheckBox->checkState() == Qt::Checked);
    d->debug = (d->ui.debugCheckBox->checkState() == Qt::Checked);
    cfg.writeEntry("cashew", d->cashew);
    cfg.writeEntry("debug", d->debug);

    emit configNeedsSaving();

    if (d->toolbox) {
        if (d->cashew) {
            d->toolbox->show();
        } else {
            d->toolbox->hide();
        }
    }
    if (d->debug) {
        updateText();
    } else {
        update();
    }
}

void ClutteredDesktop::showAppletConfigurationInterface()
{
    Applet::showConfigurationInterface();
}

// Cluttered Desktop code ends here

