/*
    Copyright (C) 2010 Kevin Krammer <kevin.krammer@gmx.at>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/

#include "akonadiagentsservice.h"

#include "agentinstancemethodjob.h"

#include <Akonadi/AgentInstance>
#include <Akonadi/AgentManager>

#include <Plasma/ServiceJob>

using namespace Akonadi;

class AkonadiAgentsService::Private
{
  AkonadiAgentsService *const q;

  public:
    explicit Private( AkonadiAgentsService *parent ) : q( parent )
    {
    }

    Private( AkonadiAgentsService *parent, const AgentInstance &agent )
      : q( parent ), mAgent( agent )
    {
    }

  public:
    const AgentInstance mAgent;
};

AkonadiAgentsService::AkonadiAgentsService( QObject *parent )
  : Plasma::Service( parent ),
    d( new Private( this ) )
{
  // make sure our config file can be found
  setName( QLatin1String( "akonadiagents" ) );
}

AkonadiAgentsService::AkonadiAgentsService( QObject *parent, QVariantList &args )
  : Plasma::Service( parent, args ),
    d( new Private( this ) )
{
  // make sure our config file can be found
  setName( QLatin1String( "akonadiagents" ) );
}

AkonadiAgentsService::AkonadiAgentsService( const AgentInstance &agent, QObject *parent )
  : Plasma::Service( parent ),
    d( new Private( this, agent ) )
{
  // make sure our config file can be found
  setName( QLatin1String( "akonadiagents" ) );

  if ( d->mAgent.type().capabilities().contains( QLatin1String( "NoConfig" ) ) ) {
    setOperationEnabled( QLatin1String( "configure" ), false );
  }

  if ( !d->mAgent.type().capabilities().contains( QLatin1String( "Resource" ) ) ) {
    setOperationEnabled( QLatin1String( "synchronizeCollectionTree" ), false );
    setOperationEnabled( QLatin1String( "synchronizeAll" ), false );
  }
}

AkonadiAgentsService::~AkonadiAgentsService()
{
  delete d;
}

Plasma::ServiceJob *AkonadiAgentsService::createJob( const QString &operation, QMap<QString, QVariant> &parameters )
{
  if ( AgentInstanceMethodJob::methods().contains( operation ) ) {
    return new AgentInstanceMethodJob( destination(), operation, parameters, this );
  }

  // If we don't know what to do for this operation, default to failing.
  return new Plasma::ServiceJob( destination(), operation, parameters, this );
}

#include "akonadiagentsservice.moc"

// kate: space-indent on; indent-width 2; replace-tabs on;
