/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   Copyright (C) 2008 by Dennis Nienhüser <earthwings@gentoo.org>        *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#ifndef KABC_CONTACT_H
#define KABC_CONTACT_H

#include <QtCore/QObject>

#include <kabc/stdaddressbook.h>

#include <contact.h>

/**
 * Represents a KABC contact (addressee)
 */
class KABCContact: public QObject, public Contact
{
Q_OBJECT

public:
	KABCContact(const KABC::Addressee &contact);

	/**
	 * @return formattedName property of KABC contact
	 */
	QString	displayName();

	/**
	 * @return Photo/Avatar of the KABC contact, or an empty string if not set
	 */
	QString photo();

	/**
	 * @return Contacts email address
	 */
	QString email();

	/**
	 * @return True if location is set in kabc
	 */
	bool hasLocation();

	/**
	 * @return Geographical contact location (longitude and latitude) if known
	 */
	QPointF location();
	
	/**
	 * @return Contacts birthday, if known, an invalid date otherwise
	 */
	QDateTime birthday();

private:
	/** KABC contact */
	const KABC::Addressee m_contact;
};

#endif // KABC_CONTACT_H
