/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   Copyright (C) 2008 by Dennis Nienhüser <earthwings@gentoo.org>        *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#include "kabcsource.h"

#include "kabccontact.h"

#include <kabc/stdaddressbook.h>

void KABCSource::initialize()
{
	update();
	connect(KABC::StdAddressBook::self(), SIGNAL(addressBookChanged(KABC::AddressBook*)), this, SLOT(update()));
}

void KABCSource::update()
{
	// Add new contacts
	QStringList contacts;
	KABC::AddressBook *ab = KABC::StdAddressBook::self();
	KABC::AddressBook::Iterator it;
	for (it = ab->begin(); it != ab->end(); ++it)
	{
		contacts.append(it->uid());
		if (!m_contacts.contains(it->uid()))
		{
			Contact::Ptr contact = Contact::Ptr(new KABCContact(*it));
			m_contacts[it->uid()] = contact;
			emit newContact(contact);
		}
		else
		{
		    emit contactChanged(m_contacts[it->uid()]);
		}
	}

	// Remove old contacts
	foreach(const QString &contact, m_contacts.keys())
	{
		if (!contacts.contains(contact))
		{
			emit contactRemoved(m_contacts[contact]);
			m_contacts.remove(contact);
		}
	}
}

#include "kabcsource.moc"

