/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   Copyright (C) 2008 by Dennis Nienhüser <earthwings@gentoo.org>        *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#include "kopetesource.h"

#include "kopete_interface.h"
#include "kopetecontact.h"

#include <QtCore/QTimer>

void KopeteSource::initialize()
{
	update();
}

void KopeteSource::update()
{
	if (QDBusConnection::sessionBus().interface()->isServiceRegistered(
			KOPETE_DBUS_SERVICE))
	{
		OrgKdeKopeteInterface kopete_interface(KOPETE_DBUS_SERVICE,
				KOPETE_DBUS_PATH, QDBusConnection::sessionBus());
		QStringList contacts = kopete_interface.contacts().value();

		// Remove old contacts
		foreach(const QString &contact, m_contacts.keys())
		{
			if (!contacts.contains(contact))
			{
				emit contactRemoved(m_contacts[contact]);
				m_contacts.remove(contact);
			}
		}

		// Add new contacts
		foreach(const QString &contact, contacts)
		{
			if (!m_contacts.contains(contact))
			{
			    KopeteContact *kopContact = new KopeteContact(contact);
				Contact::Ptr kopeteContact = Contact::Ptr(kopContact);
				m_contacts[contact] = kopeteContact;
				QObject::connect(kopContact, SIGNAL(contactChanged(KopeteContact*)),
				        this, SLOT(slotContactChanged(KopeteContact*)));
				emit newContact(kopeteContact);
			}
		}
	}

	QTimer::singleShot(5000, this, SLOT(update()));
}

void KopeteSource::slotContactChanged(KopeteContact *contact)
{
    if (m_contacts.contains(contact->id()))
    {
        emit contactChanged(m_contacts[contact->id()]);
    }
}
