/*
 *   Copyright (C) 2009 Andrew Coles <andrew_coles@yahoo.co.uk>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef LOCATIONWIFI_H
#define LOCATIONWIFI_H

#include "coordinate_backend.h"

#include <plasma/geolocation/geolocationprovider.h>

#include <QThread>
#include <QMutex>
#include <QWaitCondition>

namespace Solid {
  namespace Control {
    class WirelessNetworkInterface;
  }
}

class WiFi;

class WiFiScan : public QThread
{
    Q_OBJECT
public:
    WiFiScan(WiFi * const p);
    virtual ~WiFiScan();
    
    void update();

signals:
    void dataReady(const Plasma::DataEngine::Data &data);

protected:
    virtual void run();
    
private:
    WiFi * parent;
    QMutex m_mutex;
    QWaitCondition m_condition;
    bool m_abort;
};


class WiFi : public GeolocationProvider
{
    Q_OBJECT    
public:
    explicit WiFi(QObject *parent = 0, const QVariantList &args = QVariantList());
    virtual ~WiFi();

    virtual void update();
    
    virtual void addAccessPointData(const QString & mac, const Coordinate &);
    virtual bool saveAccessPointData();
    
private:

    friend class WiFiScan;
  
    class Private;
    Private *const d;

};

#endif
