/*
 *   Copyright (C) 2007 Ivan Cukic <ivan.cukic+kde@gmail.com>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License version 2 (or, at
 *   your option, any later version) as published by the Free Software Foundation
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "kiobrowserengine.h"

#include "plasma/datacontainer.h"
#include <kfileitem.h>
#include <kfilemetainfoitem.h>

#include <KDirWatch>
#include <KDebug>


#define InvalidIfEmpty(A) ((A.isEmpty())?(QVariant()):(QVariant(A)))
#define forMatchingSources for (DataEngine::SourceDict::iterator it = sources.begin(); it != sources.end(); it++) \
  if (dir == QDir(it.key()))

KioBrowserEngine::KioBrowserEngine(QObject* parent, const QVariantList& args) :
    Plasma::DataEngine(parent, args)
{
    kDebug() << "KioBrowserEngine::KioBrowserEngine()\n";
}

KioBrowserEngine::~KioBrowserEngine()
{
}

void KioBrowserEngine::init() {
    kDebug() << "KioBrowserEngine::init()\n";
}

bool KioBrowserEngine::sourceRequestEvent(const QString &path) {
    m_sources[path];
    setData(path, DataEngine::Data());
    updateSourceEvent(path);
    return true;
}

bool KioBrowserEngine::updateSource (const QString &source) {
    if (!m_sources.contains(source) || m_sources[source].job) return false;
    
    KIO::Job * job = createListJob(source);
    
    m_sources[source].source = source;
    m_sources[source].job = job;
    
    m_jobs[job] = & (m_sources[source]);
    
    return true;
}

void KioBrowserEngine::entries (KIO::Job *job, const KIO::UDSEntryList &list) {
    if (!m_jobs.contains(job)) return;
    KFileItem * file;
    const KIO::UDSEntryList::ConstIterator end = list.end();
    for (KIO::UDSEntryList::ConstIterator it = list.begin(); it != end; ++it) {
        file = new KFileItem(*it, m_jobs[job]->source, false, true);
        m_jobs[job]->entries.append(file);
    }
}

void KioBrowserEngine::result(KJob *job) {
    
    KIO::Job *j = (KIO::Job *) job;
    if (!m_jobs.contains(j) || !m_jobs[j]) return;

    QString source = m_jobs.take(j)->source;
    m_sources[source].job = NULL;

    QStringList keys;
    QList < QStringList > items;
    keys << "name" << "text" << "mime" << "url" << "isLocal" << "path";
    setData(source, "meta.items.fields", QVariant(keys));
    setData(source, "meta.items.count", QVariant(m_sources[source].entries.count()));
    
    int index = 0; QString prefix;
    foreach (KFileItem * file, m_sources[source].entries) {
        keys.clear();
        keys
            << file->name()
            << file->text()
            << file->mimetype()
            << file->url().url()
            << (file->url().isLocalFile() ? "true" : "false")
            << file->url().pathOrUrl();
        
        setData(source, "item." + QString::number(index++), keys);
        
        delete file;
    }
    m_sources[source].entries.clear();
}

KIO::ListJob * KioBrowserEngine::createListJob(const QString & url) {
    KIO::ListJob * job = KIO::listDir (KUrl(url), false, true);
    
    connect(
        job, SIGNAL(entries(KIO::Job *, const KIO::UDSEntryList &)),
        this, SLOT(entries(KIO::Job *, const KIO::UDSEntryList &))
    );
    
    connect(
        job, SIGNAL(result(KJob *)), 
        this, SLOT(result(KJob *))
    );
    
    return job;
}

void KioBrowserEngine::removeSource(const QString &source) {
    Plasma::DataEngine::removeSource(source);
    if (!m_sources.contains(source)) return;
    KIO::Job * job = m_sources[source].job;
    m_jobs.remove(job);
    job->kill();
    foreach(KFileItem * file, m_sources[source].entries) {
        delete file;
    }
    m_sources.remove(source);
    
}

#include "kiobrowserengine.moc"
