

#include <QtGui>
#include <plasma/animator.h>
#include <plasma/animation.h>
#include "red.h"

QAnimationGroup *setAnimations(QGraphicsWidget *widget)
{
    QSequentialAnimationGroup *sequential = new QSequentialAnimationGroup(widget);
    QParallelAnimationGroup *parallel = new QParallelAnimationGroup(sequential);

    /* First animation */
    Plasma::Animation *m_anim =  Plasma::Animator::create(Plasma::Animator::ZoomAnimation);
    m_anim->setProperty("zoom", 1.0);
    m_anim->setTargetWidget(widget);
    parallel->addAnimation(m_anim);


    /* Second animation */
    m_anim = Plasma::Animator::create(Plasma::Animator::FadeAnimation);
    m_anim->setProperty("duration", 600);
    m_anim->setProperty("startOpacity", 0.0);
    m_anim->setProperty("targetOpacity", 1.0);
    m_anim->setTargetWidget(widget);


    parallel->addAnimation(m_anim);
    sequential->addAnimation(parallel);

    /* Third animation */
    m_anim = Plasma::Animator::create(Plasma::Animator::RotationAnimation);
    m_anim->setProperty("duration", 350);
    m_anim->setProperty("axis", Qt::ZAxis);
    m_anim->setProperty("reference", Plasma::Animation::Center);
    m_anim->setProperty("angle", 360);
    m_anim->setTargetWidget(widget);
    sequential->addAnimation(m_anim);

    return sequential;
}

int main(int argc, char **argv)
{
    QApplication *app = new QApplication(argc, argv);
    QGraphicsView *view = new QGraphicsView();
    QGraphicsScene *scene = new QGraphicsScene();
    QParallelAnimationGroup *group = new QParallelAnimationGroup;
    scene->setSceneRect(0, 0, 700, 700);
    view->setScene(scene);
    RedWidget *first;
    for (int i = 0; i < 4; ++i) {
        for (int j = 0; j < 4; ++j) {
            RedWidget *widget = new RedWidget();
            if ((i == 0) && (j == 0)) {
                first = widget;
            }

            if (argc == 2) {
                widget->setIconPath(argv[1]);
            } else {
                widget->setIconPath("biggerkde.png");
            }
            scene->addItem(widget);
            //widget->setGeometry(i * 150, i * 150, 150, 150);
            widget->setGeometry(i * 150, j * 150, 150, 150);
            QAbstractAnimation *sequential = setAnimations(widget);
            QObject::connect(widget, SIGNAL(clicked()), sequential, SLOT(start()));

            group->addAnimation(sequential);
        }
    }

    QObject::connect(first, SIGNAL(clicked()), group, SLOT(start()));

    view->setViewportUpdateMode(QGraphicsView::BoundingRectViewportUpdate);
    view->setRenderHints(QPainter::Antialiasing | QPainter::SmoothPixmapTransform);

#if defined(Q_OS_SYMBIAN)
    view->showFullScreen();
#else
    view->show();
#endif

    return app->exec();
}
