/////////////////////////////////////////////////////////////////////////
// storage.h                                                           //
//                                                                     //
// Copyright (C)  2010  Brian Pritchett <batenkaitos@gmail.com>        //
//                                                                     //
// This library is free software; you can redistribute it and/or       //
// modify it under the terms of the GNU Lesser General Public          //
// License as published by the Free Software Foundation; either        //
// version 2.1 of the License, or (at your option) any later version.  //
//                                                                     //
// This library is distributed in the hope that it will be useful,     //
// but WITHOUT ANY WARRANTY; without even the implied warranty of      //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU   //
// Lesser General Public License for more details.                     //
//                                                                     //
// You should have received a copy of the GNU Lesser General Public    //
// License along with this library; if not, write to the Free Software //
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA       //
// 02110-1301  USA                                                     //
/////////////////////////////////////////////////////////////////////////

#ifndef PLASMA_STORAGE_H
#define PLASMA_STORAGE_H

#include <plasma/service.h>
#include <plasma/servicejob.h>
#include <akonadi/collection.h>

//Begin StorageJob
class StorageAkonadiJob : public Plasma::ServiceJob
{
    Q_OBJECT
public:
    StorageAkonadiJob(const QString& destination,
               const QString& operation,
               const QMap<QString, QVariant>& parameters,
               const Akonadi::Collection& dataEngineCollection,
               QObject *parent = 0);
    void jobFinished(KJob *job);
    void createSourceCollectionDone(KJob* job);
    void store();
    QVariantHash retrieve();
    Akonadi::Collection m_dataEngineCollection;
    Akonadi::Collection* m_sourceCollection;
    void foundSourceCollection(KJob* job);
    void start();

};
//End StorageJob

class StorageAkonadi : public Plasma::Service
{
    Q_OBJECT

public:
    StorageAkonadi(QObject* parent, const QVariantList& args );
    ~StorageAkonadi();

protected:
    Plasma::ServiceJob* createJob(const QString &operation, QMap<QString, QVariant> &parameters);

private:
    Akonadi::Collection m_storageCollection;
    Akonadi::Collection m_dataEngineCollection;
    QString m_serviceName;
    QString m_destination;
    void foundStorageCollection(KJob *job);
    void foundDataEngineCollection(KJob* job);
    void createDataEngineCollectionDone(KJob* job);
 
};


#endif //PLASMA_STORAGE_H
