/*
 *   Copyright 2010 Brian Pritchett <batenkaitos@gmail.com>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Library General Public License as
 *   published by the Free Software Foundation; either version 2, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
#include "akonadi_serializer_storage.h"

#include <akonadi/item.h>

#include <QtCore/qplugin.h>
#include <QVariant>
#include <QByteArray>
#include <QDataStream>

using namespace Akonadi;

bool SerializerPluginstorage::deserialize( Item& item, const QByteArray& label, 
        QIODevice& data, int version )
{
  Q_UNUSED( label );
  Q_UNUSED( version );

  QByteArray b = QByteArray::fromBase64(data.readAll());
  QDataStream ds(&b, QIODevice::ReadWrite);
  QVariantHash vh;
  ds >> vh;
  item.setPayload<QVariantHash>(vh);
  return false;
}

void SerializerPluginstorage::serialize( const Item& item, const QByteArray& label, 
        QIODevice& data, int &version )
{
  Q_UNUSED( label );
  Q_UNUSED( version );

  QByteArray b;
  QDataStream ds(&b, QIODevice::ReadWrite);
  ds << item.payload<QVariantHash>();
  data.write(b.toBase64());
}

QSet<QByteArray> SerializerPluginstorage::parts( const Item &item ) const
{
  // only need to reimplement this when implementing partial serialization
  // i.e. when using the "label" parameter of the other two methods
  return ItemSerializerPlugin::parts( item );
}

Q_EXPORT_PLUGIN2( akonadi_serializer_storage, Akonadi::SerializerPluginstorage )

#include "akonadi_serializer_storage.moc"
