/*
 *   Copyright (C) 2008 Petri Damsten <damu@iki.fi>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Library General Public License as
 *   published by the Free Software Foundation; either version 2, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "kross.h"
#include <KGlobal>
#include <QStringList>
#include <KStandardDirs>
#include <kross/core/action.h>
#include <kross/core/interpreter.h>
#include <kross/core/manager.h>
#include <KDebug>

QHash<QString, QStringList> KrossScriptEngine::m_extensions;

bool KrossScriptEngine::initScript(QObject *parent,
                                   const QString &language,
                                   const QString &name,
                                   const char* type,
                                   QObject* object,
                                   const QString& objName)
{
    const QHash<QString, QStringList>& exts = extensions();
    QString scriptFile;
    QFile file(name);

    kDebug() << language << name << exts[language];
    if (QDir::isAbsolutePath(name)) {
        scriptFile = name;
    } else {
        foreach(QString ext, exts[language]) {
            QString file = name + "." + ext;
            QString path("plasma-script/%1");
            QStringList scriptFiles =
                    KGlobal::dirs()->findAllResources("data", path.arg(file));
            if (scriptFiles.size() > 0) {
                scriptFile = scriptFiles[0];
                break;
            }
        }
    }
    if (!scriptFile.isEmpty()) {
        m_action = new Kross::Action(parent, name);
        if (m_action) {
            kDebug() << language << scriptFile;
            m_action->addObject(parent, type);
            if (object) {
                m_action->addObject(object, objName);
            }
            m_action->setFile(scriptFile);
            m_action->setInterpreter(language);
            m_action->trigger();
            return true;
        }
    }
    return false;
}

QVariant KrossScriptEngine::callFunction(const QString &name,
                                         const QVariantList &args)
{
    if (m_action) {
        return m_action->callFunction(name,args);
    }
    return QVariant();
}

const QHash<QString, QStringList>& KrossScriptEngine::extensions() const
{
    if (m_extensions.isEmpty()) {
        Kross::InterpreterInfo* info;
        QStringList list;
        QString wildcards;

        foreach(QString interpretername, Kross::Manager::self().interpreters()) {
            info = Kross::Manager::self().interpreterInfo(interpretername);
            wildcards = info->wildcard();
            wildcards.replace("*.", "");
            m_extensions[interpretername] = wildcards.split(" ");
        }
    }
    return m_extensions;
}
