/***************************************************************************
 *   Copyright (C) 2009 by Wang Hoi <zealot.hoi@gmail.com>                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#ifndef KORATBAR_H
#define KORATBAR_H

#include "koratbar_export.h"

#include "koratbar.h"
#include "koratgroup.h"

#include <kdeversion.h>
#if KDE_VERSION_MAJOR == 4 && KDE_VERSION_MINOR > 2
    #include <plasma/widgets/scrollwidget.h>
#endif

#include <QGraphicsWidget>
#include <QGraphicsLayoutItem>
#include <QGraphicsSceneResizeEvent>

namespace Korat
{
class ToolBarPrivate;
class Tab;

/**
 * @class ToolBar plasma/widgets/koratbar.h <Plasma/Widgets/ToolBar>
 *
 * @short Provides generic ribbon like toolbar(menu)
 *
 * Analog and bar meters are supported.
 *
 * Svgs can have following ids:
 * - background: Drawn first to the bottom
 *       background can be a FrameSvg
 * - label0, label1, ...: Rectangles mark the label places
 * - bar: Bar for the bar meter
 *      can be replaced with bar-active and bar-inactive FrameSvg
 * - pointer: Pointer for analog meter
 * - rotatecenter: Marks the place of pointer rotation center
 * - rotateminmax: Width and height of this object are the Min and Max rotate
 *                 angles for the pointer
 * - foreground: Is drawn to top
 *
 * @author Wang Hoi
 */
class KORATBAR_EXPORT ToolBar : public QGraphicsWidget
{
    Q_OBJECT
public:
    /**
     * Constructor
     * @param parent the QGraphicsItem this koratbar is parented to.
     */
    explicit ToolBar(QGraphicsItem *parent=0);

    /**
     * Destructor
     */
    virtual ~ToolBar();

    void addTab(Tab *tab);
    void removeTab(Tab *tab);
    Tab *tabAt(int index);

protected Q_SLOTS:
    void setCurrentTab(int index);

protected:

    virtual void resizeEvent(QGraphicsSceneResizeEvent * event);

    /**
     * Reimplemented from QGraphicsWidget
     */
    virtual void paint(QPainter *painter,
                       const QStyleOptionGraphicsItem *option,
                       QWidget *widget = 0);

private:
    ToolBarPrivate *const d;

    Q_PRIVATE_SLOT(d, void slidingCompleted(QGraphicsItem *item))
};

} // namespace Korat

//#include "koratbar.moc"

#endif // KORATBAR_H
