#ifndef NODEITEM_H
#define NODEITEM_H

#include "edgesource.h"

#include <QColor>
#include <QGraphicsItem>
#include <QPainter>
#include <QStyleOptionGraphicsItem>
#include <QWidget>

#include "node.h"

class Edge;
class NodeItem : public QObject,
                 public EdgeSource
{
    Q_OBJECT
public:
    NodeItem();
    ~NodeItem();

    virtual void paint(QPainter * painter,
                       const QStyleOptionGraphicsItem * option,
                       QWidget * widget = 0);
    virtual QRectF boundingRect() const;
    virtual QPainterPath shape() const;

    void setNode(Quasar::Node *filter);
    Quasar::Node *node() const
    {
        return m_node;
    }

    void setBackgroundColor(const QColor &color)
    { m_bgColor = color; }
    QColor backgroundColor() const { return m_bgColor; }

    QString propertyAt(const QPointF &pt) const;
    QRectF propertyRect(const QString &prop) const;

    enum { Type = UserType + 3 };
    int type() const { return Type; }

protected:
    virtual void mousePressEvent(QGraphicsSceneMouseEvent *event);
    virtual void mouseReleaseEvent(QGraphicsSceneMouseEvent *event);
    virtual void mouseMoveEvent(QGraphicsSceneMouseEvent *event);
    virtual void contextMenuEvent(QGraphicsSceneContextMenuEvent *event);
    void updateSize();

    void createBoxConnections(QPainterPath &path,
                              const QList<Quasar::Property> &props);
private:
    QRectF           m_rect;
    Quasar::Node    *m_node;
    Edge            *m_sourceEdge;
    Edge            *m_destEdge;
    QColor           m_bgColor;

    QPainterPath          m_baseShape;
    QPainterPath          m_shape;
    QMap<QString, QRectF> m_propLocations;

    QString m_clickedProperty;
};

#endif
