#include "composition.h"
#include "texture.h"
#include "node.h"

#include <QApplication>
#include <QGLWidget>
#include <QGLPixelBuffer>
#include <QDebug>

using namespace Quasar;

static bool filter(const QString &loc)
{
    QImage img(loc);
    if (img.isNull()) {
        qWarning()<<"Image '"<< loc <<"' is null!";
        return true;
    }


    qDebug()<<"Filters = "<<Composition::groups();
    Composition compo;
    Node *input = compo.createNode("/input/file");
    Q_ASSERT(input);
    input->setPropertyValue("filename", loc);

    Node *node = compo.createNode("/color/saturation");
    Q_ASSERT(node);
    node->setPropertyValue("alpha", 4.0);
    Quasar::link(input, "texture", node, "texture");

    Node *outNode = compo.createNode("/output/image");
    Q_ASSERT(outNode);
    Quasar::link(node, "output", outNode, "texture");

    compo.prepare();
    compo.execute();

    QVariant res = outNode->propertyValue("image");
    img = qvariant_cast<QImage>(res);

    qDebug()<<"Img is null: "<<img.isNull();
    img.save("out.png");

    return img.isNull();
}

int main(int argc, char **argv)
{
    QApplication app(argc, argv);

    if (argc != 2) {
        qWarning("Missing arg: \"%s filename\"\n",
                 argv[0]);
        exit(1);
    }
    QStringList groups = Composition::groups();
    foreach(QString group, groups) {
        qDebug()<<"Filter group = "<<group;
        qDebug()<<"\tnodes: "<<Composition::nodesInGroup(group);
    }

    return filter(argv[1]);
}
