#include "imageinput_p.h"

#include "texture.h"

#include <QGLWidget>
#include <QImage>
#include <qdebug.h>

namespace Quasar
{

ImageInputNode::ImageInputNode()
    : GraphicsNode("File Input")
{
    Quasar::Property prop = {
        Quasar::Property::InputProperty,

        QString::fromLatin1("filename"),
        QVariant::String,
        QString(),
        QString::fromLatin1("File to load"),

        QString(),
        QVariant(),
        QVariant(),
    };
    addProperty(QString::fromLatin1("filename"),
                prop);

    Quasar::Property out = {
        Quasar::Property::OutputProperty,

        QString::fromLatin1("texture"),
        QVariant::UserType,
        QVariant(),
        QString::fromLatin1("Loaded texture"),

        QVariant(),
        QVariant(),
        QVariant(),
    };
    addProperty(QString::fromLatin1("texture"),
                out);
}

bool ImageInputNode::execute()
{
    if (m_texture.isNull())
        m_texture =  Texture((const char*)m_image.bits(),
                             m_image.width(), m_image.height(), GL_RGBA);
    outputReady(QString::fromLatin1("texture"), m_texture);

    if (m_texture.isNull())
        qWarning()<<"\tImageInputNode ouput texture is null";
    return !m_texture.isNull();
}

void ImageInputNode::prepare()
{
}

QRectF ImageInputNode::domainOfDefinition(const QRectF &rect) const
{
    return QRectF(0, 0, m_image.width(), m_image.height());
}

QRectF ImageInputNode::regionOfInterest(const QRectF &rect) const
{
    return rect;
}

void ImageInputNode::propertyChanged(const QString &name,
                                     const QVariant &val)
{
    if (name == QLatin1String("filename")) {
        QString filename = propertyValue(QString::fromLatin1("filename")).toString();
        m_image = QImage(filename);
        m_image = QGLWidget::convertToGLFormat(m_image);
        m_texture = Texture();//null it
    }
}

}



