#define GL_GLEXT_PROTOTYPES
#include "transformnode_p.h"

#include <QTransform>
#include <QVariant>
#include <QDebug>

#include <GL/glext.h>
#include <GL/gl.h>

using namespace Quasar;


static const char *vertShaderText =
    "uniform mat4 userMatrix;\n"
    "void main() {\n"
    "   gl_FrontColor = gl_Color;\n"
    "   gl_TexCoord[0] = gl_MultiTexCoord0;\n"
    "   gl_Position = gl_ModelViewProjectionMatrix * (userMatrix * gl_Vertex);\n"
    "}\n";

static const char *fragShaderText =
    "const vec3 lumCoeff = vec3(0.2125, 0.7154, 0.0721);\n"
    "uniform sampler2D srcTex;\n"
    "void main() {\n"
    "    gl_FragColor =  texture2D(srcTex,  gl_TexCoord[0].st);\n"
    "}\n";

TransformNode::TransformNode()
    : FilterNode("Transformation")
{
    setShaderSources(vertShaderText, fragShaderText);
    Quasar::Property aprop = {
        Quasar::Property::InputProperty,

        QString::fromLatin1("userMatrix"),
        QVariant::Transform,
        QTransform(),
        QString::fromLatin1("Transformation matrix"),

        QTransform(),
        QTransform(),
        QTransform(),
    };
    addProperty(QString::fromLatin1("userMatrix"), aprop);
}

QRectF TransformNode::domainOfDefinition(const QRectF &rect) const
{
    QVariant res = propertyValue("userMatrix");
    QTransform trans = qvariant_cast<QTransform>(res);
    QPolygonF poly = trans.map(QPolygonF(rect));
    return poly.boundingRect();
}

QRectF TransformNode::regionOfInterest(const QRectF &rect) const
{
    return rect;
}

void TransformNode::programChanged(GLuint prog)
{
    GLuint texLoc = glGetUniformLocationARB(prog, "srcTex");
    glUniform1iARB(texLoc, 0);
    QVariant var = propertyValue("matrix");
    QTransform trans = qvariant_cast<QTransform>(var);
    setShaderMatrixUniform("userMatrix", trans);
}

