
#include <QtTest/QtTest>
#include "node.h"

#include "testnodes.h"

class tst_Nodes : public QObject
{
    Q_OBJECT

public:
    tst_Nodes();
    virtual ~tst_Nodes();


public slots:
    void init();
    void cleanup();
private slots:
    void test_setting();
    void test_connections();
    void test_connections2();
    void test_dependencies();
};


tst_Nodes::tst_Nodes()
{
}

tst_Nodes::~tst_Nodes()
{
}

void tst_Nodes::init()
{
    // No initialisation is required
}

void tst_Nodes::cleanup()
{
    // No cleanup is required.
}

void tst_Nodes::test_setting()
{
    Quasar::Node *dummy = new DummyAddNode();

    QCOMPARE(dummy->isDirty(), true);
    QCOMPARE(dummy->setPropertyValue("null", 4), false);
    QVERIFY(dummy->propertyValue("null") == QVariant());

    QCOMPARE(dummy->propertyValue("number1").toDouble(), 0.);
    QCOMPARE(dummy->setPropertyValue("number1", 5), true);
    QCOMPARE(dummy->propertyValue("number1").toDouble(), 5.);

    QCOMPARE(dummy->setPropertyValue("number1", -1), false);
    QCOMPARE(dummy->propertyValue("number1").toDouble(),
             dummy->property("number1").min.toDouble());
    QCOMPARE(dummy->setPropertyValue("number1", 11), false);
    QCOMPARE(dummy->propertyValue("number1").toDouble(),
             dummy->property("number1").max.toDouble());

    QCOMPARE(dummy->isDirty(), true);
    dummy->setPropertyValue("number2", 3);
    QCOMPARE(dummy->process(), true);
    QCOMPARE(dummy->propertyValue("result").toDouble(), 13.);
    QCOMPARE(dummy->isDirty(), false);

    dummy->setPropertyValue("number1", 2);
    QCOMPARE(dummy->isDirty(), true);
    QCOMPARE(dummy->process(), true);
    QCOMPARE(dummy->propertyValue("result").toDouble(), 5.);
    QCOMPARE(dummy->isDirty(), false);

    dummy->setPropertyValue("number1", 9);
    dummy->setPropertyValue("number2", 8);
    QCOMPARE(dummy->propertyValue("number1").toDouble(), 9.);
    QCOMPARE(dummy->propertyValue("number2").toDouble(), 8.);
    QCOMPARE(dummy->isDirty(), true);
    QCOMPARE(dummy->process(), true);
    QCOMPARE(dummy->propertyValue("result").toDouble(), 17.);
    QCOMPARE(dummy->isDirty(), false);


    QCOMPARE(dummy->setPropertyValue("result", 3), false);
    QCOMPARE(dummy->propertyValue("result").toDouble(), 17.);
    QCOMPARE(dummy->isDirty(), false);
    QCOMPARE(dummy->process(), true);
    QCOMPARE(dummy->propertyValue("result").toDouble(), 17.);
    QCOMPARE(dummy->setPropertyValue("result", 12), false);
    QCOMPARE(dummy->propertyValue("result").toDouble(), 17.);
    QCOMPARE(dummy->isDirty(), false);

    delete dummy;
}

void tst_Nodes::test_connections()
{
    Quasar::Node *dummy1 = new DummyAddNode();
    Quasar::Node *dummy2 = new DummyAddNode();
    Quasar::Node *dummy3 = new DummyAddNode();

    QCOMPARE(dummy1->outputNodes().count(), 0);
    QCOMPARE(dummy1->inputNodes().count(), 0);
    QCOMPARE(dummy2->outputNodes().count(), 0);
    QCOMPARE(dummy2->inputNodes().count(), 0);
    QCOMPARE(Quasar::link(dummy1, "result",
                          dummy3, "number1"), true);
    QCOMPARE(Quasar::link(dummy2, "result",
                          dummy3, "number1"), true);

    QCOMPARE(dummy1->hasInputs(), false);
    QCOMPARE(dummy2->hasInputs(), false);
    QCOMPARE(dummy1->hasOutputs(), false);
    QCOMPARE(dummy2->hasOutputs(), true);
    QCOMPARE(dummy3->hasInputs(), true);
    QCOMPARE(dummy3->hasOutputs(), false);

    QCOMPARE(dummy1->outputNodes().count(), 0);
    QCOMPARE(dummy1->inputNodes().count(), 0);
    QCOMPARE(dummy2->outputNodes().count(), 1);
    QCOMPARE(dummy2->inputNodes().count(), 0);
    QCOMPARE(dummy3->outputNodes().count(), 0);
    QCOMPARE(dummy3->inputNodes().count(), 1);

    QCOMPARE(Quasar::link(dummy1, "result",
                          dummy3, "number2"), true);

    QCOMPARE(dummy1->outputNodes().count(), 1);
    QCOMPARE(dummy1->inputNodes().count(), 0);
    QCOMPARE(dummy2->outputNodes().count(), 1);
    QCOMPARE(dummy2->inputNodes().count(), 0);
    QCOMPARE(dummy3->outputNodes().count(), 0);
    QCOMPARE(dummy3->inputNodes().count(), 2);

    QCOMPARE(Quasar::link(dummy1, "result",
                          dummy3, "number1"), true);
    QCOMPARE(Quasar::link(dummy2, "result",
                          dummy3, "number2"), true);

    QCOMPARE(dummy1->outputNodes().count(), 1);
    QCOMPARE(dummy1->inputNodes().count(), 0);
    QCOMPARE(dummy2->outputNodes().count(), 1);
    QCOMPARE(dummy2->inputNodes().count(), 0);
    QCOMPARE(dummy3->outputNodes().count(), 0);
    QCOMPARE(dummy3->inputNodes().count(), 2);

    QCOMPARE(Quasar::unlink(dummy1, "result",
                            dummy3, "number1"), false);

    QCOMPARE(dummy1->outputNodes().count(), 1);
    QCOMPARE(dummy1->inputNodes().count(), 0);
    QCOMPARE(dummy3->outputNodes().count(), 0);
    QCOMPARE(dummy3->inputNodes().count(), 2);

    QCOMPARE(Quasar::unlink(dummy2, "result",
                            dummy3, "number2"), true);

    QCOMPARE(dummy2->outputNodes().count(), 0);
    QCOMPARE(dummy2->inputNodes().count(), 0);
    QCOMPARE(dummy3->outputNodes().count(), 0);
    QCOMPARE(dummy3->inputNodes().count(), 1);

    QCOMPARE(Quasar::unlink(dummy1, "result",
                            dummy3, "number2"), true);
    QCOMPARE(dummy1->outputNodes().count(), 0);
    QCOMPARE(dummy1->inputNodes().count(), 0);
    QCOMPARE(dummy2->outputNodes().count(), 0);
    QCOMPARE(dummy2->inputNodes().count(), 0);
    QCOMPARE(dummy3->outputNodes().count(), 0);
    QCOMPARE(dummy3->inputNodes().count(), 0);

    QCOMPARE(Quasar::link(dummy1, "result",
                          dummy3, "number1"), true);
    QCOMPARE(Quasar::link(dummy1, "result",
                          dummy2, "number1"), true);
    QCOMPARE(Quasar::link(dummy2, "result",
                          dummy3, "number2"), true);
    QCOMPARE(dummy1->outputNodes().count(), 2);
    QCOMPARE(dummy1->inputNodes().count(), 0);
    QCOMPARE(dummy2->outputNodes().count(), 1);
    QCOMPARE(dummy2->inputNodes().count(), 1);
    QCOMPARE(dummy3->outputNodes().count(), 0);
    QCOMPARE(dummy3->inputNodes().count(), 2);

    QCOMPARE(dummy1->unlink("result"), true);
    QCOMPARE(dummy1->outputNodes().count(), 0);
    QCOMPARE(dummy1->inputNodes().count(), 0);
    QCOMPARE(dummy2->outputNodes().count(), 1);
    QCOMPARE(dummy2->inputNodes().count(), 0);
    QCOMPARE(dummy3->outputNodes().count(), 0);
    QCOMPARE(dummy3->inputNodes().count(), 1);

    QCOMPARE(Quasar::link(dummy2, "result",
                          dummy1, "number2"), true);
    QCOMPARE(dummy2->unlink(), true);
    QCOMPARE(dummy1->outputNodes().count(), 0);
    QCOMPARE(dummy1->inputNodes().count(), 0);
    QCOMPARE(dummy2->outputNodes().count(), 0);
    QCOMPARE(dummy2->inputNodes().count(), 0);
    QCOMPARE(dummy3->outputNodes().count(), 0);
    QCOMPARE(dummy3->inputNodes().count(), 0);

    delete dummy1;
    delete dummy2;
    delete dummy3;
}

void tst_Nodes::test_connections2()
{
    Quasar::Node *dummy1 = new DummyAddNode();
    Quasar::Node *dummy2 = new DummyAddNode();
    Quasar::Node *dummy3 = new DummyAddNode();

    QCOMPARE(Quasar::link(dummy1, "result",
                          dummy2, "number1"), true);
    QCOMPARE(Quasar::link(dummy1, "result",
                          dummy2, "number2"), true);
    QCOMPARE(Quasar::link(dummy2, "result",
                          dummy3, "number1"), true);
    QCOMPARE(Quasar::link(dummy2, "result",
                          dummy3, "number2"), true);

    QCOMPARE(dummy1->outputNodes().count(), 2);
    QCOMPARE(dummy1->inputNodes().count(), 0);
    QCOMPARE(dummy2->outputNodes().count(), 2);
    QCOMPARE(dummy2->inputNodes().count(), 2);
    QCOMPARE(dummy3->outputNodes().count(), 0);
    QCOMPARE(dummy3->inputNodes().count(), 2);

    dummy2->unlink();
    QCOMPARE(dummy1->outputNodes().count(), 0);
    QCOMPARE(dummy1->inputNodes().count(), 0);
    QCOMPARE(dummy2->outputNodes().count(), 0);
    QCOMPARE(dummy2->inputNodes().count(), 0);
    QCOMPARE(dummy3->outputNodes().count(), 0);
    QCOMPARE(dummy3->inputNodes().count(), 0);

    QCOMPARE(Quasar::link(dummy1, "result",
                          dummy2, "number1"), true);
    QCOMPARE(Quasar::link(dummy1, "result",
                          dummy2, "number2"), true);
    QCOMPARE(Quasar::link(dummy2, "result",
                          dummy3, "number1"), true);
    QCOMPARE(Quasar::link(dummy2, "result",
                          dummy3, "number2"), true);

    QCOMPARE(dummy1->outputNodes().count(), 2);
    QCOMPARE(dummy1->inputNodes().count(), 0);
    QCOMPARE(dummy2->outputNodes().count(), 2);
    QCOMPARE(dummy2->inputNodes().count(), 2);
    QCOMPARE(dummy3->outputNodes().count(), 0);
    QCOMPARE(dummy3->inputNodes().count(), 2);

    delete dummy2;
    QCOMPARE(dummy1->outputNodes().count(), 0);
    QCOMPARE(dummy1->inputNodes().count(), 0);
    QCOMPARE(dummy3->outputNodes().count(), 0);
    QCOMPARE(dummy3->inputNodes().count(), 0);

    delete dummy1;
    delete dummy3;
}

void tst_Nodes::test_dependencies()
{
    Quasar::Node *dummy1 = new DummyAddNode();
    Quasar::Node *dummy2 = new DummyAddNode();
    Quasar::Node *dummy3 = new DummyAddNode();

    QCOMPARE(dummy1->hasInputs(), false);
    QCOMPARE(dummy2->hasInputs(), false);
    QCOMPARE(dummy3->hasInputs(), false);
    QCOMPARE(dummy1->hasOutputs(), false);
    QCOMPARE(dummy2->hasOutputs(), false);
    QCOMPARE(dummy3->hasOutputs(), false);

    QCOMPARE(Quasar::link(dummy1, "result",
                          dummy2, "number1"), true);
    QCOMPARE(Quasar::link(dummy1, "result",
                          dummy2, "number2"), true);
    QCOMPARE(Quasar::link(dummy1, "result",
                          dummy3, "number1"), true);
    QCOMPARE(Quasar::link(dummy1, "result",
                          dummy3, "number2"), true);

    QCOMPARE(dummy1->hasInputs(), false);
    QCOMPARE(dummy2->hasInputs(), true);
    QCOMPARE(dummy3->hasInputs(), true);
    QCOMPARE(dummy1->hasOutputs(), true);
    QCOMPARE(dummy2->hasOutputs(), false);
    QCOMPARE(dummy3->hasOutputs(), false);

    QCOMPARE(dummy1->isDependentOn(dummy2), false);
    QCOMPARE(dummy1->isDependentOn(dummy3), false);
    QCOMPARE(dummy2->isDependentOn(dummy1), true);
    QCOMPARE(dummy3->isDependentOn(dummy1), true);
    QCOMPARE(dummy2->isDependentOn(dummy3), false);

    Quasar::Node *dummy4 = new DummyAddNode();
    Quasar::Node *dummy5 = new DummyAddNode();
    QCOMPARE(Quasar::link(dummy2, "result",
                          dummy4, "number1"), true);
    QCOMPARE(Quasar::link(dummy3, "result",
                          dummy4, "number2"), true);

    QCOMPARE(dummy4->isDependentOn(dummy1), true);
    QCOMPARE(dummy4->isDependentOn(dummy2), true);
    QCOMPARE(dummy4->isDependentOn(dummy3), true);
    QCOMPARE(dummy1->isDependentOn(dummy4), false);
    QCOMPARE(dummy2->isDependentOn(dummy4), false);
    QCOMPARE(dummy3->isDependentOn(dummy4), false);

    QCOMPARE(Quasar::link(dummy2, "result",
                          dummy5, "number1"), true);
    QCOMPARE(Quasar::link(dummy2, "result",
                          dummy5, "number2"), true);
    QCOMPARE(dummy5->isDependentOn(dummy1), true);
    QCOMPARE(dummy5->isDependentOn(dummy2), true);
    QCOMPARE(dummy5->isDependentOn(dummy3), false);
    QCOMPARE(dummy5->isDependentOn(dummy4), false);
    QCOMPARE(dummy2->isDependentOn(dummy5), false);
    QCOMPARE(dummy1->isDependentOn(dummy5), false);
    QCOMPARE(dummy4->isDependentOn(dummy5), false);

    Quasar::Node *dummy6 = new DummyAddNode();
    QCOMPARE(Quasar::link(dummy5, "result",
                          dummy6, "number1"), true);
    QCOMPARE(Quasar::link(dummy5, "result",
                          dummy6, "number2"), true);
    QCOMPARE(dummy6->isDependentOn(dummy5), true);
    QCOMPARE(dummy6->isDependentOn(dummy2), true);
    QCOMPARE(dummy6->isDependentOn(dummy1), true);
    QCOMPARE(dummy6->isDependentOn(dummy6), false);
    QCOMPARE(dummy6->isDependentOn(dummy3), false);
    QCOMPARE(dummy6->isDependentOn(dummy4), false);
}

QTEST_APPLESS_MAIN(tst_Nodes)


#include "tst_nodes.moc"
