#define GL_GLEXT_PROTOTYPES
#include "simplefilter.h"

#include <GL/glext.h>
#include <GL/gl.h>

static const char *vertShaderText =
    "void main() {\n"
    "   gl_FrontColor = gl_Color;\n"
    "   gl_TexCoord[0] = gl_MultiTexCoord0;\n"
    "   gl_Position = gl_ModelViewProjectionMatrix * gl_Vertex;\n"
    "}\n";

static const char *fragShaderText =
    "uniform sampler2D srcTex;\n"
    "void main() {\n"
    "    vec4 frag = mix(texture2D(srcTex,  gl_TexCoord[0].st), gl_Color, 0.8);\n"
    "    gl_FragColor = frag;\n"
    "}\n";

using namespace Quasar;

SimpleFilter::SimpleFilter()
    : FilterNode("sample")
{
}

QRectF SimpleFilter::domainOfDefinition(const QRectF &rect) const
{
    return QRectF();
}

QRectF SimpleFilter::regionOfInterest(const QRectF &rect) const
{
    return QRectF();
}

bool SimpleFilter::execute()
{
    glUseProgram(programId());
}

void SimpleFilter::prepare()
{
    loadShaders(QString::fromLatin1(vertShaderText),
                QString::fromLatin1(fragShaderText));
}

void SimpleFilter::programChanged(GLuint prog)
{
    GLuint texLoc = glGetUniformLocationARB(programId(), "srcTex");
    glUniform1iARB(texLoc, 0);
}
