/*
    This file is part of the KDE project.
    Copyright (c) 2005 Friedrich W. H. Kossebau <Friedrich.W.H@Kossebau.de>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License version 2 as published by the Free Software Foundation.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301  USA
*/


#ifndef KLOCALSESSIONMANAGER_H
#define KLOCALSESSIONMANAGER_H

// qt specific
#include <qobject.h>
// kde specific
#include <kconfig.h>
#include <kapplication.h>


class QWidget;


struct KSession {
  /** name of the display */
  QString DisplayName;
  QString Origin;
  /** login name of the user */
  QString LoginName;
  /** Name of the session */
  QString Session;
  /** id of the vt */
  int VTId;
  /** true if this is the current session */
  bool Current: 1;
  /** true if a plain terminal */
  bool IsTty: 1;
};

typedef QValueList<KSession> KSessionList;


class KLocalSessionManager : public QObject
{
  Q_OBJECT

  public:
    /**
     * @param S screen widget to place the dialogs on
     * @param SN screen number
     */
    KLocalSessionManager( QWidget *S, int SN );

  public:
    bool canCreateSession() const;
    bool canSwitchSession() const;
    bool canLogoutSession() const;
    bool canLockSession() const;
    bool canSaveSession() const;
    bool canShutdown() const;

    int numberOfReservedSessions() const;
    bool getBootOptions( QStringList &Options, int &DefaultOption, int &CurrentOption ) const;
    bool doesLockOnSwitch() const;
    bool getLocalSessions( KSessionList &List );
    KSessionList localSessions();

  public slots:
    void createSession() const;
    void switchToSession( int SessionId ) const;
    void saveSession() const;
    void lockSession() const;
    void logoutSession() const;
    void shutdown( KApplication::ShutdownType ShutdownType,
                    KApplication::ShutdownMode ShutdownMode,
                    const QString &BootOption = QString::null );

    void setLockOnSwitch( bool Lock );


  protected:
    /** pointer to the screen widget to place the dialogs on */
    QWidget *Screen;
    int ScreenNumber;
    bool LockOnSwitch;
};

#endif
